"""
Manage basic template commands
"""
import codecs
import io
import logging
import os
import time

import salt.utils.data
import salt.utils.files
import salt.utils.sanitizers
import salt.utils.stringio
import salt.utils.versions

log = logging.getLogger(__name__)


# FIXME: we should make the default encoding of a .sls file a configurable
#        option in the config, and default it to 'utf-8'.
#
SLS_ENCODING = "utf-8"  # this one has no BOM.
SLS_ENCODER = codecs.getencoder(SLS_ENCODING)


def compile_template(
    template,
    renderers,
    default,
    blacklist,
    whitelist,
    saltenv="base",
    sls="",
    input_data="",
    context=None,
    **kwargs
):
    """
    Take the path to a template and return the high data structure
    derived from the template.

    Helpers:

    :param mask_value:
        Mask value for debugging purposes (prevent sensitive information etc)
        example: "mask_value="pass*". All "passwd", "password", "pass" will
        be masked (as text).
    """

    # if any error occurs, we return an empty dictionary
    ret = {}

    log.debug("compile template: %s", template)

    if "env" in kwargs:
        # "env" is not supported; Use "saltenv".
        kwargs.pop("env")

    if template != ":string:":
        # Template was specified incorrectly
        if not isinstance(template, str):
            log.error("Template was specified incorrectly: %s", template)
            return ret
        # Template does not exist
        if not os.path.isfile(template):
            log.error("Template does not exist: %s", template)
            return ret
        # Template is an empty file
        if salt.utils.files.is_empty(template):
            log.debug("Template is an empty file: %s", template)
            return ret

        with codecs.open(template, encoding=SLS_ENCODING) as ifile:
            # data input to the first render function in the pipe
            input_data = ifile.read()
            if not input_data.strip():
                # Template is nothing but whitespace
                log.error("Template is nothing but whitespace: %s", template)
                return ret

    # Get the list of render funcs in the render pipe line.
    render_pipe = template_shebang(
        template, renderers, default, blacklist, whitelist, input_data
    )

    windows_newline = "\r\n" in input_data

    input_data = io.StringIO(input_data)
    for render, argline in render_pipe:
        if salt.utils.stringio.is_readable(input_data):
            input_data.seek(0)  # pylint: disable=no-member
        render_kwargs = dict(renderers=renderers, tmplpath=template)
        if context:
            render_kwargs["context"] = context
        render_kwargs.update(kwargs)
        if argline:
            render_kwargs["argline"] = argline
        start = time.time()
        ret = render(input_data, saltenv, sls, **render_kwargs)
        log.profile(
            "Time (in seconds) to render '%s' using '%s' renderer: %s",
            template,
            render.__module__.split(".")[-1],
            time.time() - start,
        )
        if ret is None:
            # The file is empty or is being written elsewhere
            time.sleep(0.01)
            ret = render(input_data, saltenv, sls, **render_kwargs)
        input_data = ret
        if log.isEnabledFor(logging.GARBAGE):  # pylint: disable=no-member
            # If ret is not a StringIO (which means it was rendered using
            # yaml, mako, or another engine which renders to a data
            # structure) we don't want to log this.
            if salt.utils.stringio.is_readable(ret):
                log.debug(
                    "Rendered data from file: %s:\n%s",
                    template,
                    salt.utils.sanitizers.mask_args_value(
                        salt.utils.data.decode(ret.read()), kwargs.get("mask_value")
                    ),
                )  # pylint: disable=no-member
                ret.seek(0)  # pylint: disable=no-member

    # Preserve newlines from original template
    if windows_newline:
        if salt.utils.stringio.is_readable(ret):
            is_stringio = True
            contents = ret.read()
        else:
            is_stringio = False
            contents = ret

        if isinstance(contents, str):
            if "\r\n" not in contents:
                contents = contents.replace("\n", "\r\n")
                ret = io.StringIO(contents) if is_stringio else contents
            else:
                if is_stringio:
                    ret.seek(0)
    return ret


def compile_template_str(template, renderers, default, blacklist, whitelist):
    """
    Take template as a string and return the high data structure
    derived from the template.
    """
    fn_ = salt.utils.files.mkstemp()
    with salt.utils.files.fopen(fn_, "wb") as ofile:
        ofile.write(SLS_ENCODER(template)[0])
    return compile_template(fn_, renderers, default, blacklist, whitelist)


def template_shebang(template, renderers, default, blacklist, whitelist, input_data):
    """
    Check the template shebang line and return the list of renderers specified
    in the pipe.

    Example shebang lines::

      #!yaml_jinja
      #!yaml_mako
      #!mako|yaml
      #!jinja|yaml
      #!jinja|mako|yaml
      #!mako|yaml|stateconf
      #!jinja|yaml|stateconf
      #!mako|yaml_odict
      #!mako|yaml_odict|stateconf

    """
    line = ""
    # Open up the first line of the sls template
    if template == ":string:":
        line = input_data.split()[0]
    else:
        with salt.utils.files.fopen(template, "r") as ifile:
            line = salt.utils.stringutils.to_unicode(ifile.readline())

    # Check if it starts with a shebang and not a path
    if line.startswith("#!") and not line.startswith("#!/"):
        # pull out the shebang data
        # If the shebang does not contain recognized/not-blacklisted/whitelisted
        # renderers, do not fall back to the default renderer
        return check_render_pipe_str(line.strip()[2:], renderers, blacklist, whitelist)
    else:
        return check_render_pipe_str(default, renderers, blacklist, whitelist)


# A dict of combined renderer (i.e., rend1_rend2_...) to
# render-pipe (i.e., rend1|rend2|...)
#
OLD_STYLE_RENDERERS = {}

for comb in (
    "yaml_jinja",
    "yaml_mako",
    "yaml_wempy",
    "json_jinja",
    "json_mako",
    "json_wempy",
    "yamlex_jinja",
    "yamlexyamlex_mako",
    "yamlexyamlex_wempy",
):

    fmt, tmpl = comb.split("_")
    OLD_STYLE_RENDERERS[comb] = "{}|{}".format(tmpl, fmt)


def check_render_pipe_str(pipestr, renderers, blacklist, whitelist):
    """
    Check that all renderers specified in the pipe string are available.
    If so, return the list of render functions in the pipe as
    (render_func, arg_str) tuples; otherwise return [].
    """
    if pipestr is None:
        return []
    parts = [r.strip() for r in pipestr.split("|")]
    # Note: currently, | is not allowed anywhere in the shebang line except
    #       as pipes between renderers.

    results = []
    try:
        if parts[0] == pipestr and pipestr in OLD_STYLE_RENDERERS:
            parts = OLD_STYLE_RENDERERS[pipestr].split("|")
        for part in parts:
            name, argline = (part + " ").split(" ", 1)
            if whitelist and name not in whitelist or blacklist and name in blacklist:
                log.warning(
                    'The renderer "%s" is disallowed by configuration and '
                    "will be skipped.",
                    name,
                )
                continue
            results.append((renderers[name], argline.strip()))
        return results
    except KeyError:
        log.error('The renderer "%s" is not available', pipestr)
        return []



"""test for pillar hg_pillar.py"""


import os
import shutil
import subprocess
import tempfile

import salt.pillar.hg_pillar as hg_pillar
import salt.utils.files
import salt.utils.yaml
from tests.support.mixins import (
    AdaptedConfigurationTestCaseMixin,
    LoaderModuleMockMixin,
)
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import TestCase, skipIf

COMMIT_USER_NAME = "test_user"
# file contents
PILLAR_CONTENT = {"gna": "hello"}
FILE_DATA = {"top.sls": {"base": {"*": ["user"]}}, "user.sls": PILLAR_CONTENT}


HGLIB = hg_pillar.hglib


@skipIf(HGLIB is None, "python-hglib library not installed")
class HgPillarTestCase(
    TestCase, AdaptedConfigurationTestCaseMixin, LoaderModuleMockMixin
):
    "test hg_pillar pillar"
    maxDiff = None

    def setup_loader_modules(self):
        self.tmpdir = tempfile.mkdtemp(dir=RUNTIME_VARS.TMP)
        self.addCleanup(shutil.rmtree, self.tmpdir)
        cachedir = os.path.join(self.tmpdir, "cachedir")
        os.makedirs(os.path.join(cachedir, "hg_pillar"))
        self.hg_repo_path = self._create_hg_repo()
        return {
            hg_pillar: {
                "__opts__": {
                    "cachedir": cachedir,
                    "pillar_roots": {},
                    "file_roots": {},
                    "state_top": "top.sls",
                    "extension_modules": "",
                    "renderer": "yaml_jinja",
                    "pillar_opts": False,
                    "renderer_blacklist": [],
                    "renderer_whitelist": [],
                    "file_ignore_glob": [],
                    "file_ignore_regex": [],
                }
            }
        }

    def _create_hg_repo(self):
        "create repo in tempdir"
        hg_repo = os.path.join(self.tmpdir, "repo_pillar")
        os.makedirs(hg_repo)
        subprocess.check_call(["hg", "init", hg_repo])
        for filename in FILE_DATA:
            with salt.utils.files.fopen(
                os.path.join(hg_repo, filename), "w"
            ) as data_file:
                salt.utils.yaml.safe_dump(FILE_DATA[filename], data_file)
        subprocess.check_call(
            [
                "hg",
                "ci",
                "-A",
                "-R",
                hg_repo,
                "-m",
                "first commit",
                "-u",
                COMMIT_USER_NAME,
            ]
        )
        return hg_repo

    def test_base(self):
        "check hg repo is imported correctly"
        mypillar = hg_pillar.ext_pillar(
            "*", None, "file://{}".format(self.hg_repo_path)
        )
        self.assertEqual(PILLAR_CONTENT, mypillar)



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

import salt.modules.data as data
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, mock_open, patch
from tests.support.unit import TestCase


class DataTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.data
    """

    def setup_loader_modules(self):
        return {data: {}}

    # 'clear' function tests: 1

    def test_clear(self):
        """
        Test if it clear out all of the data in the minion datastore
        """
        with patch("os.remove", MagicMock(return_value="")):
            with patch.dict(data.__opts__, {"cachedir": ""}):
                self.assertTrue(data.clear())

    # 'load' function tests: 1

    def test_load(self):
        """
        Test if it return all of the data in the minion datastore
        """
        with patch("salt.payload.Serial.load", MagicMock(return_value=True)):
            mocked_fopen = MagicMock(return_value=True)
            mocked_fopen.__enter__ = MagicMock(return_value=mocked_fopen)
            mocked_fopen.__exit__ = MagicMock()
            with patch("salt.utils.files.fopen", MagicMock(return_value=mocked_fopen)):
                with patch("salt.payload.loads", MagicMock(return_value=True)):
                    with patch.dict(data.__opts__, {"cachedir": "/"}):
                        self.assertTrue(data.load())

    # 'dump' function tests: 3

    def test_dump(self):
        """
        Test if it replace the entire datastore with a passed data structure
        """
        with patch.dict(data.__opts__, {"cachedir": "/"}):
            with patch("salt.utils.files.fopen", mock_open()):
                self.assertTrue(data.dump('{"eggs": "spam"}'))

    def test_dump_isinstance(self):
        """
        Test if it replace the entire datastore with a passed data structure
        """
        with patch("ast.literal_eval", MagicMock(return_value="")):
            self.assertFalse(data.dump("salt"))

    def test_dump_ioerror(self):
        """
        Test if it replace the entire datastore with a passed data structure
        """
        with patch.dict(data.__opts__, {"cachedir": "/"}):
            mock = MagicMock(side_effect=IOError(""))
            with patch("salt.utils.files.fopen", mock):
                self.assertFalse(data.dump('{"eggs": "spam"}'))

    # 'update' function tests: 1

    def test_update(self):
        """
        Test if it update a key with a value in the minion datastore
        """
        with patch("salt.modules.data.load", MagicMock(return_value={})), patch(
            "salt.modules.data.dump", MagicMock(return_value=True)
        ):
            self.assertTrue(data.update("foo", "salt"))

    # 'get' function tests: 2

    def test_get(self):
        """
        Test if it gets a value from the minion datastore
        """
        with patch("salt.modules.data.load", MagicMock(return_value={"salt": "SALT"})):
            self.assertEqual(data.get("salt"), "SALT")

    def test_get_vals(self):
        """
        Test if it gets values from the minion datastore
        """
        with patch(
            "salt.modules.data.load",
            MagicMock(return_value={"salt": "SALT", "salt1": "SALT1"}),
        ):
            self.assertEqual(data.get(["salt", "salt1"]), ["SALT", "SALT1"])

    # 'cas' function tests: 1

    def test_cas_not_load(self):
        """
        Test if it check and set a value in the minion datastore
        """
        with patch(
            "salt.modules.data.load",
            MagicMock(return_value={"salt": "SALT", "salt1": "SALT1"}),
        ):
            self.assertFalse(data.cas("salt3", "SALT", "SALTSTACK"))

    def test_cas_not_equal(self):
        """
        Test if it check and set a value in the minion datastore
        """
        with patch(
            "salt.modules.data.load",
            MagicMock(return_value={"salt": "SALT", "salt1": "SALT1"}),
        ):
            self.assertFalse(data.cas("salt", "SALT", "SALTSTACK"))

    def test_cas(self):
        """
        Test if it check and set a value in the minion datastore
        """
        with patch(
            "salt.modules.data.load",
            MagicMock(return_value={"salt": "SALT", "salt1": "SALT1"}),
        ), patch("salt.modules.data.dump", MagicMock(return_value=True)):
            self.assertTrue(data.cas("salt", "SALTSTACK", "SALT"))



"""
Linux File Access Control Lists

The Linux ACL state module requires the `getfacl` and `setfacl` binaries.

Ensure a Linux ACL is present

.. code-block:: yaml

     root:
       acl.present:
         - name: /root
         - acl_type: user
         - acl_name: damian
         - perms: rwx

Ensure a Linux ACL does not exist

.. code-block:: yaml

     root:
       acl.absent:
         - name: /root
         - acl_type: user
         - acl_name: damian
         - perms: rwx

Ensure a Linux ACL list is present

.. code-block:: yaml

     root:
       acl.list_present:
         - name: /root
         - acl_type: user
         - acl_names:
           - damian
           - homer
         - perms: rwx

Ensure a Linux ACL list does not exist

.. code-block:: yaml

     root:
       acl.list_absent:
         - name: /root
         - acl_type: user
         - acl_names:
           - damian
           - homer
         - perms: rwx
"""


import logging
import os

import salt.utils.path
from salt.exceptions import CommandExecutionError

log = logging.getLogger(__name__)

__virtualname__ = "acl"


def __virtual__():
    """
    Ensure getfacl & setfacl exist
    """
    if salt.utils.path.which("getfacl") and salt.utils.path.which("setfacl"):
        return __virtualname__

    return (
        False,
        "The linux_acl state cannot be loaded: the getfacl or setfacl binary is not in"
        " the path.",
    )


def present(name, acl_type, acl_name="", perms="", recurse=False, force=False):
    """
    Ensure a Linux ACL is present

    name
        The acl path

    acl_type
        The type of the acl is used for it can be 'user' or 'group'

    acl_name
        The  user or group

    perms
        Set the permissions eg.: rwx

    recurse
        Set the permissions recursive in the path

    force
        Wipe out old permissions and ensure only the new permissions are set
    """
    ret = {"name": name, "result": True, "changes": {}, "comment": ""}

    _octal = {"r": 4, "w": 2, "x": 1, "-": 0}
    _octal_lookup = {0: "-", 1: "r", 2: "w", 4: "x"}

    if not os.path.exists(name):
        ret["comment"] = "{} does not exist".format(name)
        ret["result"] = False
        return ret

    __current_perms = __salt__["acl.getfacl"](name, recursive=recurse)

    if acl_type.startswith(("d:", "default:")):
        _acl_type = ":".join(acl_type.split(":")[1:])
        _current_perms = __current_perms[name].get("defaults", {})
        _default = True
    else:
        _acl_type = acl_type
        _current_perms = __current_perms[name]
        _default = False

    # The getfacl execution module lists default with empty names as being
    # applied to the user/group that owns the file, e.g.,
    # default:group::rwx would be listed as default:group:root:rwx
    # In this case, if acl_name is empty, we really want to search for root
    # but still uses '' for other

    # We search through the dictionary getfacl returns for the owner of the
    # file if acl_name is empty.
    if acl_name == "":
        _search_name = __current_perms[name].get("comment").get(_acl_type, "")
    else:
        _search_name = acl_name

    if _current_perms.get(_acl_type, None) or _default:
        try:
            user = [
                i
                for i in _current_perms[_acl_type]
                if next(iter(i.keys())) == _search_name
            ].pop()
        except (AttributeError, IndexError, StopIteration, KeyError):
            user = None

        if user:
            octal_sum = sum([_octal.get(i, i) for i in perms])
            need_refresh = False
            # If recursive check all paths retrieved via acl.getfacl
            if recurse:
                for path in __current_perms:
                    acl_found = False
                    if _default:
                        # Recusive default acls only apply to directories
                        if not os.path.isdir(path):
                            continue
                        _current_perms_path = __current_perms[path].get("defaults", {})
                    else:
                        _current_perms_path = __current_perms[path]
                    for user_acl in _current_perms_path.get(_acl_type, []):
                        if (
                            _search_name in user_acl
                            and user_acl[_search_name]["octal"] == octal_sum
                        ):
                            acl_found = True
                    if not acl_found:
                        need_refresh = True
                        break

            # Check the permissions from the already located file
            elif user[_search_name]["octal"] == sum([_octal.get(i, i) for i in perms]):
                need_refresh = False
            # If they don't match then refresh
            else:
                need_refresh = True

            if not need_refresh:
                ret["comment"] = "Permissions are in the desired state"
            else:
                _num = user[_search_name]["octal"]
                new_perms = "{}{}{}".format(
                    _octal_lookup[_num & 1],
                    _octal_lookup[_num & 2],
                    _octal_lookup[_num & 4],
                )
                changes = {
                    "new": {"acl_name": acl_name, "acl_type": acl_type, "perms": perms},
                    "old": {
                        "acl_name": acl_name,
                        "acl_type": acl_type,
                        "perms": new_perms,
                    },
                }

                if __opts__["test"]:
                    ret.update(
                        {
                            "comment": (
                                "Updated permissions will be applied for "
                                "{}: {} -> {}".format(acl_name, new_perms, perms)
                            ),
                            "result": None,
                            "changes": changes,
                        }
                    )
                    return ret
                try:
                    if force:
                        __salt__["acl.wipefacls"](
                            name, recursive=recurse, raise_err=True
                        )

                    __salt__["acl.modfacl"](
                        acl_type,
                        acl_name,
                        perms,
                        name,
                        recursive=recurse,
                        raise_err=True,
                    )
                    ret.update(
                        {
                            "comment": "Updated permissions for {}".format(acl_name),
                            "result": True,
                            "changes": changes,
                        }
                    )
                except CommandExecutionError as exc:
                    ret.update(
                        {
                            "comment": "Error updating permissions for {}: {}".format(
                                acl_name, exc.strerror
                            ),
                            "result": False,
                        }
                    )
        else:
            changes = {
                "new": {"acl_name": acl_name, "acl_type": acl_type, "perms": perms}
            }

            if __opts__["test"]:
                ret.update(
                    {
                        "comment": "New permissions will be applied for {}: {}".format(
                            acl_name, perms
                        ),
                        "result": None,
                        "changes": changes,
                    }
                )
                ret["result"] = None
                return ret

            try:
                if force:
                    __salt__["acl.wipefacls"](name, recursive=recurse, raise_err=True)

                __salt__["acl.modfacl"](
                    acl_type, acl_name, perms, name, recursive=recurse, raise_err=True
                )
                ret.update(
                    {
                        "comment": "Applied new permissions for {}".format(acl_name),
                        "result": True,
                        "changes": changes,
                    }
                )
            except CommandExecutionError as exc:
                ret.update(
                    {
                        "comment": "Error updating permissions for {}: {}".format(
                            acl_name, exc.strerror
                        ),
                        "result": False,
                    }
                )

    else:
        ret["comment"] = "ACL Type does not exist"
        ret["result"] = False

    return ret


def absent(name, acl_type, acl_name="", perms="", recurse=False):
    """
    Ensure a Linux ACL does not exist

    name
        The acl path

    acl_type
        The type of the acl is used for, it can be 'user' or 'group'

    acl_name
        The user or group

    perms
        Remove the permissions eg.: rwx

    recurse
        Set the permissions recursive in the path
    """
    ret = {"name": name, "result": True, "changes": {}, "comment": ""}

    if not os.path.exists(name):
        ret["comment"] = "{} does not exist".format(name)
        ret["result"] = False
        return ret

    __current_perms = __salt__["acl.getfacl"](name, recursive=recurse)

    if acl_type.startswith(("d:", "default:")):
        _acl_type = ":".join(acl_type.split(":")[1:])
        _current_perms = __current_perms[name].get("defaults", {})
        _default = True
    else:
        _acl_type = acl_type
        _current_perms = __current_perms[name]
        _default = False

    # The getfacl execution module lists default with empty names as being
    # applied to the user/group that owns the file, e.g.,
    # default:group::rwx would be listed as default:group:root:rwx
    # In this case, if acl_name is empty, we really want to search for root
    # but still uses '' for other

    # We search through the dictionary getfacl returns for the owner of the
    # file if acl_name is empty.
    if acl_name == "":
        _search_name = __current_perms[name].get("comment").get(_acl_type, "")
    else:
        _search_name = acl_name

    if _current_perms.get(_acl_type, None) or _default:
        try:
            user = [
                i
                for i in _current_perms[_acl_type]
                if next(iter(i.keys())) == _search_name
            ].pop()
        except (AttributeError, IndexError, StopIteration, KeyError):
            user = None

        need_refresh = False
        for path in __current_perms:
            acl_found = False
            for user_acl in __current_perms[path].get(_acl_type, []):
                if _search_name in user_acl:
                    acl_found = True
                    break
            if acl_found:
                need_refresh = True
                break

        if user or need_refresh:
            ret["comment"] = "Removing permissions"

            if __opts__["test"]:
                ret["result"] = None
                return ret

            __salt__["acl.delfacl"](acl_type, acl_name, perms, name, recursive=recurse)
        else:
            ret["comment"] = "Permissions are in the desired state"

    else:
        ret["comment"] = "ACL Type does not exist"
        ret["result"] = False

    return ret


def list_present(name, acl_type, acl_names=None, perms="", recurse=False, force=False):
    """
    Ensure a Linux ACL list is present

    Takes a list of acl names and add them to the given path

    name
        The acl path

    acl_type
        The type of the acl is used for it can be 'user' or 'group'

    acl_names
        The list of users or groups

    perms
        Set the permissions eg.: rwx

    recurse
        Set the permissions recursive in the path

    force
        Wipe out old permissions and ensure only the new permissions are set
    """
    if acl_names is None:
        acl_names = []
    ret = {"name": name, "result": True, "changes": {}, "comment": ""}

    _octal = {"r": 4, "w": 2, "x": 1, "-": 0}
    _octal_perms = sum([_octal.get(i, i) for i in perms])
    if not os.path.exists(name):
        ret["comment"] = "{} does not exist".format(name)
        ret["result"] = False
        return ret

    __current_perms = __salt__["acl.getfacl"](name)

    if acl_type.startswith(("d:", "default:")):
        _acl_type = ":".join(acl_type.split(":")[1:])
        _current_perms = __current_perms[name].get("defaults", {})
        _default = True
    else:
        _acl_type = acl_type
        _current_perms = __current_perms[name]
        _default = False
        _origin_group = _current_perms.get("comment", {}).get("group", None)
        _origin_owner = _current_perms.get("comment", {}).get("owner", None)

        _current_acl_types = []
        diff_perms = False
        for key in _current_perms[acl_type]:
            for current_acl_name in key.keys():
                _current_acl_types.append(current_acl_name.encode("utf-8"))
                diff_perms = _octal_perms == key[current_acl_name]["octal"]
        if acl_type == "user":
            try:
                _current_acl_types.remove(_origin_owner)
            except ValueError:
                pass
        else:
            try:
                _current_acl_types.remove(_origin_group)
            except ValueError:
                pass
        diff_acls = set(_current_acl_types) ^ set(acl_names)
        if not diff_acls and diff_perms and not force:
            ret = {
                "name": name,
                "result": True,
                "changes": {},
                "comment": "Permissions and {}s are in the desired state".format(
                    acl_type
                ),
            }
            return ret
    # The getfacl execution module lists default with empty names as being
    # applied to the user/group that owns the file, e.g.,
    # default:group::rwx would be listed as default:group:root:rwx
    # In this case, if acl_names is empty, we really want to search for root
    # but still uses '' for other

    # We search through the dictionary getfacl returns for the owner of the
    # file if acl_names is empty.
    if acl_names == "":
        _search_names = __current_perms[name].get("comment").get(_acl_type, "")
    else:
        _search_names = acl_names

    if _current_perms.get(_acl_type, None) or _default:
        try:
            users = {}
            for i in _current_perms[_acl_type]:
                if i and next(iter(i.keys())) in _search_names:
                    users.update(i)
        except (AttributeError, KeyError):
            users = None

        if users:
            changes = {}
            for count, search_name in enumerate(_search_names):
                if search_name in users:
                    if users[search_name]["octal"] == sum(
                        [_octal.get(i, i) for i in perms]
                    ):
                        ret["comment"] = "Permissions are in the desired state"
                    else:
                        changes.update(
                            {
                                "new": {
                                    "acl_name": ", ".join(acl_names),
                                    "acl_type": acl_type,
                                    "perms": _octal_perms,
                                },
                                "old": {
                                    "acl_name": ", ".join(acl_names),
                                    "acl_type": acl_type,
                                    "perms": str(users[search_name]["octal"]),
                                },
                            }
                        )
                        if __opts__["test"]:
                            ret.update(
                                {
                                    "comment": (
                                        "Updated permissions will be applied for "
                                        "{}: {} -> {}".format(
                                            acl_names,
                                            str(users[search_name]["octal"]),
                                            perms,
                                        )
                                    ),
                                    "result": None,
                                    "changes": changes,
                                }
                            )
                            return ret
                        try:
                            if force:
                                __salt__["acl.wipefacls"](
                                    name, recursive=recurse, raise_err=True
                                )

                            for acl_name in acl_names:
                                __salt__["acl.modfacl"](
                                    acl_type,
                                    acl_name,
                                    perms,
                                    name,
                                    recursive=recurse,
                                    raise_err=True,
                                )
                            ret.update(
                                {
                                    "comment": "Updated permissions for {}".format(
                                        acl_names
                                    ),
                                    "result": True,
                                    "changes": changes,
                                }
                            )
                        except CommandExecutionError as exc:
                            ret.update(
                                {
                                    "comment": (
                                        "Error updating permissions for {}: {}".format(
                                            acl_names, exc.strerror
                                        )
                                    ),
                                    "result": False,
                                }
                            )
                else:
                    changes = {
                        "new": {
                            "acl_name": ", ".join(acl_names),
                            "acl_type": acl_type,
                            "perms": perms,
                        }
                    }

                    if __opts__["test"]:
                        ret.update(
                            {
                                "comment": (
                                    "New permissions will be applied for {}: {}".format(
                                        acl_names, perms
                                    )
                                ),
                                "result": None,
                                "changes": changes,
                            }
                        )
                        ret["result"] = None
                        return ret

                    try:
                        if force:
                            __salt__["acl.wipefacls"](
                                name, recursive=recurse, raise_err=True
                            )

                        for acl_name in acl_names:
                            __salt__["acl.modfacl"](
                                acl_type,
                                acl_name,
                                perms,
                                name,
                                recursive=recurse,
                                raise_err=True,
                            )
                        ret.update(
                            {
                                "comment": "Applied new permissions for {}".format(
                                    ", ".join(acl_names)
                                ),
                                "result": True,
                                "changes": changes,
                            }
                        )
                    except CommandExecutionError as exc:
                        ret.update(
                            {
                                "comment": (
                                    "Error updating permissions for {}: {}".format(
                                        acl_names, exc.strerror
                                    )
                                ),
                                "result": False,
                            }
                        )

        else:
            changes = {
                "new": {
                    "acl_name": ", ".join(acl_names),
                    "acl_type": acl_type,
                    "perms": perms,
                }
            }

            if __opts__["test"]:
                ret.update(
                    {
                        "comment": "New permissions will be applied for {}: {}".format(
                            acl_names, perms
                        ),
                        "result": None,
                        "changes": changes,
                    }
                )
                ret["result"] = None
                return ret

            try:
                if force:
                    __salt__["acl.wipefacls"](name, recursive=recurse, raise_err=True)

                for acl_name in acl_names:
                    __salt__["acl.modfacl"](
                        acl_type,
                        acl_name,
                        perms,
                        name,
                        recursive=recurse,
                        raise_err=True,
                    )
                ret.update(
                    {
                        "comment": "Applied new permissions for {}".format(
                            ", ".join(acl_names)
                        ),
                        "result": True,
                        "changes": changes,
                    }
                )
            except CommandExecutionError as exc:
                ret.update(
                    {
                        "comment": "Error updating permissions for {}: {}".format(
                            acl_names, exc.strerror
                        ),
                        "result": False,
                    }
                )

    else:
        ret["comment"] = "ACL Type does not exist"
        ret["result"] = False

    return ret


def list_absent(name, acl_type, acl_names=None, recurse=False):
    """
    Ensure a Linux ACL list does not exist

    Takes a list of acl names and remove them from the given path

    name
        The acl path

    acl_type
        The type of the acl is used for, it can be 'user' or 'group'

    acl_names
        The list of users or groups

    perms
        Remove the permissions eg.: rwx

    recurse
        Set the permissions recursive in the path

    """
    if acl_names is None:
        acl_names = []

    ret = {"name": name, "result": True, "changes": {}, "comment": ""}

    if not os.path.exists(name):
        ret["comment"] = "{} does not exist".format(name)
        ret["result"] = False
        return ret

    __current_perms = __salt__["acl.getfacl"](name)

    if acl_type.startswith(("d:", "default:")):
        _acl_type = ":".join(acl_type.split(":")[1:])
        _current_perms = __current_perms[name].get("defaults", {})
        _default = True
    else:
        _acl_type = acl_type
        _current_perms = __current_perms[name]
        _default = False
    # The getfacl execution module lists default with empty names as being
    # applied to the user/group that owns the file, e.g.,
    # default:group::rwx would be listed as default:group:root:rwx
    # In this case, if acl_names is empty, we really want to search for root
    # but still uses '' for other

    # We search through the dictionary getfacl returns for the owner of the
    # file if acl_names is empty.
    if not acl_names:
        _search_names = set(__current_perms[name].get("comment").get(_acl_type, ""))
    else:
        _search_names = set(acl_names)

    if _current_perms.get(_acl_type, None) or _default:
        try:
            users = {}
            for i in _current_perms[_acl_type]:
                if i and next(iter(i.keys())) in _search_names:
                    users.update(i)
        except (AttributeError, KeyError):
            users = None

        if users:
            ret["comment"] = "Removing permissions"

            if __opts__["test"]:
                ret["result"] = None
                return ret
            for acl_name in acl_names:
                __salt__["acl.delfacl"](acl_type, acl_name, name, recursive=recurse)
        else:
            ret["comment"] = "Permissions are in the desired state"

    else:
        ret["comment"] = "ACL Type does not exist"
        ret["result"] = False

    return ret



"""
    Log4Mongo Logging Handler
    =========================

    This module provides a logging handler for sending salt logs to MongoDB

    Configuration
    -------------

    In the salt configuration file (e.g. /etc/salt/{master,minion}):

    .. code-block:: yaml

        log4mongo_handler:
          host: mongodb_host
          port: 27017
          database_name: logs
          collection: salt_logs
          username: logging
          password: reindeerflotilla
          write_concern: 0
          log_level: warning


    Log Level
    .........

    If not set, the log_level will be set to the level defined in the global
    configuration file setting.

    .. admonition:: Inspiration

        This work was inspired by the Salt logging handlers for LogStash and
        Sentry and by the log4mongo Python implementation.
"""

import logging
import socket

from salt.log.mixins import NewStyleClassMixIn
from salt.log.setup import LOG_LEVELS

try:
    from log4mongo.handlers import MongoHandler, MongoFormatter

    HAS_MONGO = True
except ImportError:
    HAS_MONGO = False

__virtualname__ = "mongo"


def __virtual__():
    if not HAS_MONGO:
        return False
    return __virtualname__


class FormatterWithHost(logging.Formatter, NewStyleClassMixIn):
    def format(self, record):
        mongoformatter = MongoFormatter()
        document = mongoformatter.format(record)
        document["hostname"] = socket.gethostname()
        return document


def setup_handlers():
    handler_id = "log4mongo_handler"
    if handler_id in __opts__:
        config_fields = {
            "host": "host",
            "port": "port",
            "database_name": "database_name",
            "collection": "collection",
            "username": "username",
            "password": "password",
            "write_concern": "w",
        }

        config_opts = {}
        for config_opt, arg_name in config_fields.items():
            config_opts[arg_name] = __opts__[handler_id].get(config_opt)

        config_opts["level"] = LOG_LEVELS[
            __opts__[handler_id].get("log_level", __opts__.get("log_level", "error"))
        ]

        handler = MongoHandler(formatter=FormatterWithHost(), **config_opts)
        yield handler
    else:
        yield False



"""
Support for Apache

Please note: The functions in here are SUSE-specific. Placing them in this
separate file will allow them to load only on SUSE systems, while still
loading under the ``apache`` namespace.
"""

import logging

import salt.utils.path

log = logging.getLogger(__name__)

__virtualname__ = "apache"


def __virtual__():
    """
    Only load the module if apache is installed.
    """
    if salt.utils.path.which("apache2ctl") and __grains__["os_family"] == "Suse":
        return __virtualname__
    return (False, "apache execution module not loaded: apache not installed.")


def check_mod_enabled(mod):
    """
    Checks to see if the specific apache mod is enabled.

    This will only be functional on operating systems that support
    `a2enmod -l` to list the enabled mods.

    CLI Example:

    .. code-block:: bash

        salt '*' apache.check_mod_enabled status
    """
    if mod.endswith(".load") or mod.endswith(".conf"):
        mod_name = mod[:-5]
    else:
        mod_name = mod

    cmd = "a2enmod -l"
    try:
        active_mods = __salt__["cmd.run"](cmd, python_shell=False).split(" ")
    except Exception as e:  # pylint: disable=broad-except
        return e

    return mod_name in active_mods


def a2enmod(mod):
    """
    Runs a2enmod for the given mod.

    CLI Example:

    .. code-block:: bash

        salt '*' apache.a2enmod vhost_alias
    """
    ret = {}
    command = ["a2enmod", mod]

    try:
        status = __salt__["cmd.retcode"](command, python_shell=False)
    except Exception as e:  # pylint: disable=broad-except
        return e

    ret["Name"] = "Apache2 Enable Mod"
    ret["Mod"] = mod

    if status == 1:
        ret["Status"] = "Mod {} Not found".format(mod)
    elif status == 0:
        ret["Status"] = "Mod {} enabled".format(mod)
    else:
        ret["Status"] = status

    return ret


def a2dismod(mod):
    """
    Runs a2dismod for the given mod.

    CLI Example:

    .. code-block:: bash

        salt '*' apache.a2dismod vhost_alias
    """
    ret = {}
    command = ["a2dismod", mod]

    try:
        status = __salt__["cmd.retcode"](command, python_shell=False)
    except Exception as e:  # pylint: disable=broad-except
        return e

    ret["Name"] = "Apache2 Disable Mod"
    ret["Mod"] = mod

    if status == 256:
        ret["Status"] = "Mod {} Not found".format(mod)
    elif status == 0:
        ret["Status"] = "Mod {} disabled".format(mod)
    else:
        ret["Status"] = status

    return ret



"""
    :codeauthor: Eric Radman <ericshane@eradman.com>
"""


import pytest
import salt.modules.openbsdpkg as openbsdpkg
from tests.support.mock import MagicMock, call, patch


class ListPackages:
    def __init__(self):
        self._iteration = 0

    def __call__(self):
        pkg_lists = [
            {"vim": "7.4.1467p1-gtk2"},
            {"png": "1.6.23", "vim": "7.4.1467p1-gtk2", "ruby": "2.3.1p1"},
        ]
        pkgs = pkg_lists[self._iteration]
        self._iteration += 1
        return pkgs


@pytest.fixture
def configure_loader_modules():
    return {openbsdpkg: {}}


def test_list_pkgs():
    """
    Test for listing installed packages.
    """

    def _add_data(data, key, value):
        data[key] = value

    pkg_info_out = [
        "png-1.6.23",
        "vim-7.4.1467p1-gtk2",  # vim--gtk2
        "ruby-2.3.1p1",  # ruby%2.3
    ]
    run_stdout_mock = MagicMock(return_value="\n".join(pkg_info_out))
    patches = {
        "cmd.run_stdout": run_stdout_mock,
        "pkg_resource.add_pkg": _add_data,
        "pkg_resource.sort_pkglist": MagicMock(),
        "pkg_resource.stringify": MagicMock(),
    }
    with patch.dict(openbsdpkg.__salt__, patches):
        pkgs = openbsdpkg.list_pkgs()
        assert pkgs == {"png": "1.6.23", "vim--gtk2": "7.4.1467p1", "ruby": "2.3.1p1"}
    run_stdout_mock.assert_called_once_with("pkg_info -q -a", output_loglevel="trace")


def test_install_pkgs():
    """
    Test package install behavior for the following conditions:
    - only base package name is given ('png')
    - a flavor is specified ('vim--gtk2')
    - a branch is specified ('ruby%2.3')
    """

    parsed_targets = (
        {"vim--gtk2": None, "png": None, "ruby%2.3": None},
        "repository",
    )
    cmd_out = {
        "retcode": 0,
        "stdout": "quirks-2.241 signed on 2016-07-26T16:56:10Z",
        "stderr": "",
    }
    run_all_mock = MagicMock(return_value=cmd_out)
    patches = {
        "cmd.run_all": run_all_mock,
        "pkg_resource.parse_targets": MagicMock(return_value=parsed_targets),
        "pkg_resource.stringify": MagicMock(),
        "pkg_resource.sort_pkglist": MagicMock(),
    }

    with patch.dict(openbsdpkg.__salt__, patches):
        with patch("salt.modules.openbsdpkg.list_pkgs", ListPackages()):
            added = openbsdpkg.install()
            expected = {
                "png": {"new": "1.6.23", "old": ""},
                "ruby": {"new": "2.3.1p1", "old": ""},
            }
            assert added == expected
    expected_calls = [
        call("pkg_add -x -I png--%", output_loglevel="trace", python_shell=False),
        call("pkg_add -x -I ruby--%2.3", output_loglevel="trace", python_shell=False),
        call("pkg_add -x -I vim--gtk2%", output_loglevel="trace", python_shell=False),
    ]
    run_all_mock.assert_has_calls(expected_calls, any_order=True)
    assert run_all_mock.call_count == 3


def test_list_pkgs_no_context():
    """
    Test for listing installed packages.
    """

    def _add_data(data, key, value):
        data[key] = value

    pkg_info_out = [
        "png-1.6.23",
        "vim-7.4.1467p1-gtk2",  # vim--gtk2
        "ruby-2.3.1p1",  # ruby%2.3
    ]
    run_stdout_mock = MagicMock(return_value="\n".join(pkg_info_out))
    patches = {
        "cmd.run_stdout": run_stdout_mock,
        "pkg_resource.add_pkg": _add_data,
        "pkg_resource.sort_pkglist": MagicMock(),
        "pkg_resource.stringify": MagicMock(),
    }
    with patch.dict(openbsdpkg.__salt__, patches), patch.object(
        openbsdpkg, "_list_pkgs_from_context"
    ) as list_pkgs_context_mock:
        pkgs = openbsdpkg.list_pkgs(use_context=False)
        list_pkgs_context_mock.assert_not_called()
        list_pkgs_context_mock.reset_mock()

        pkgs = openbsdpkg.list_pkgs(use_context=False)
        list_pkgs_context_mock.assert_not_called()
        list_pkgs_context_mock.reset_mock()


def test_upgrade_available():
    """
    Test upgrade_available when an update is available.
    """
    ret = MagicMock(return_value="5.4.2p0")
    with patch("salt.modules.openbsdpkg.latest_version", ret):
        assert openbsdpkg.upgrade_available("zsh")


def test_upgrade_not_available():
    """
    Test upgrade_available when an update is not available.
    """
    ret = MagicMock(return_value="")
    with patch("salt.modules.openbsdpkg.latest_version", ret):
        assert not openbsdpkg.upgrade_available("zsh")


def test_upgrade():
    """
    Test upgrading packages.
    """
    ret = {}
    pkg_add_u_stdout = [
        "quirks-2.402 signed on 2018-01-02T16:30:59Z",
        "Read shared items: ok",
    ]
    ret["stdout"] = "\n".join(pkg_add_u_stdout)
    ret["retcode"] = 0
    run_all_mock = MagicMock(return_value=ret)
    with patch.dict(openbsdpkg.__salt__, {"cmd.run_all": run_all_mock}):
        with patch("salt.modules.openbsdpkg.list_pkgs", ListPackages()):
            upgraded = openbsdpkg.upgrade()
            expected = {
                "png": {"new": "1.6.23", "old": ""},
                "ruby": {"new": "2.3.1p1", "old": ""},
            }
            assert upgraded == expected



from salt.pillar import extra_minion_data_in_pillar
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock
from tests.support.unit import TestCase


class ExtraMinionDataInPillarTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.pillar.extra_minion_data_in_pillar
    """

    def setup_loader_modules(self):
        return {extra_minion_data_in_pillar: {}}

    def setUp(self):
        self.pillar = MagicMock()
        self.extra_minion_data = {
            "key1": {"subkey1": "value1"},
            "key2": {"subkey2": {"subsubkey2": "value2"}},
            "key3": "value3",
            "key4": {"subkey4": "value4"},
        }

    def test_extra_values_none_or_empty(self):
        ret = extra_minion_data_in_pillar.ext_pillar(
            "fake_id", self.pillar, "fake_include", None
        )
        self.assertEqual(ret, {})
        ret = extra_minion_data_in_pillar.ext_pillar(
            "fake_id", self.pillar, "fake_include", {}
        )
        self.assertEqual(ret, {})

    def test_include_all(self):
        for include_all in ["*", "<all>"]:
            ret = extra_minion_data_in_pillar.ext_pillar(
                "fake_id", self.pillar, include_all, self.extra_minion_data
            )
            self.assertEqual(ret, self.extra_minion_data)

    def test_include_specific_keys(self):
        # Tests partially existing key, key with and without subkey,
        ret = extra_minion_data_in_pillar.ext_pillar(
            "fake_id",
            self.pillar,
            include=["key1:subkey1", "key2:subkey3", "key3", "key4"],
            extra_minion_data=self.extra_minion_data,
        )
        self.assertEqual(
            ret,
            {
                "key1": {"subkey1": "value1"},
                "key3": "value3",
                "key4": {"subkey4": "value4"},
            },
        )



import pytest

pytestmark = [
    pytest.mark.windows_whitelisted,
]


def test_jinja_renderer_argline(state, state_tree):
    """
    This is a test case for https://github.com/saltstack/salt/issues/55124
    """
    renderer_contents = """

    import salt.utils.stringio


    def render(gpg_data, saltenv="base", sls="", argline="", **kwargs):
        '''
        Renderer which returns the text value of the SLS file, instead of a
        StringIO object.
        '''
        if salt.utils.stringio.is_readable(gpg_data):
            return gpg_data.getvalue()
        else:
            return gpg_data
    """
    sls_contents = """
    #!issue55124|jinja -s|yaml

    'Who am I?':
      cmd.run:
        - name: echo {{ salt.cmd.run('whoami') }}
    """
    with pytest.helpers.temp_file(
        "issue51499.py", renderer_contents, state_tree / "_renderers"
    ), pytest.helpers.temp_file("issue-55124.sls", sls_contents, state_tree):
        ret = state.sls("issue-55124")
        for state_return in ret:
            assert state_return.result is True



import pytest
import salt.ext.tornado.iostream
import salt.transport.ipc
import salt.utils.asynchronous
import salt.utils.platform
from saltfactories.utils.ports import get_unused_localhost_port


def test_ipc_connect_in_async_methods():
    "The connect method is in IPCMessageSubscriber's async_methods property"
    assert "connect" in salt.transport.ipc.IPCMessageSubscriber.async_methods


async def test_ipc_connect_sync_wrapped(io_loop, tmp_path):
    """
    Ensure IPCMessageSubscriber.connect gets wrapped by
    salt.utils.asynchronous.SyncWrapper.
    """
    if salt.utils.platform.is_windows():
        socket_path = get_unused_localhost_port()
    else:
        socket_path = str(tmp_path / "noexist.ipc")
    subscriber = salt.utils.asynchronous.SyncWrapper(
        salt.transport.ipc.IPCMessageSubscriber,
        args=(socket_path,),
        kwargs={"io_loop": io_loop},
        loop_kwarg="io_loop",
    )
    with pytest.raises(salt.ext.tornado.iostream.StreamClosedError):
        # Don't `await subscriber.connect()`, that's the purpose of the SyncWrapper
        subscriber.connect()



import pytest
import salt.modules.postgres as postgres

# 'md5' + md5('password' + 'username')
md5_pw = "md55a231fcdb710d73268c4f44283487ba2"

scram_pw = (
    "SCRAM-SHA-256$4096:wLr5nqC+3F+r7FdQPnB+nA==$"
    "0hn08ZdX8kirGaL4TM0j13digH9Wl365OOzCtAuF2pE=:"
    "LzAh/MGUdjYkdbDzcOKpfGwa3WwPUsyGcY+TEnSpcto="
)


def idfn(val):
    if val == md5_pw:
        return "md5_pw"
    if val == scram_pw:
        return "scram_pw"


@pytest.mark.parametrize(
    "role,password,verifier,method,result",
    [
        ("username", "password", md5_pw, "md5", True),
        ("another", "password", md5_pw, "md5", False),
        ("username", "another", md5_pw, "md5", False),
        ("username", md5_pw, md5_pw, "md5", True),
        ("username", "md5another", md5_pw, "md5", False),
        ("username", "password", md5_pw, True, True),
        ("another", "password", md5_pw, True, False),
        ("username", "another", md5_pw, True, False),
        ("username", md5_pw, md5_pw, True, True),
        ("username", "md5another", md5_pw, True, False),
        (None, "password", scram_pw, "scram-sha-256", True),
        (None, "another", scram_pw, "scram-sha-256", False),
        (None, scram_pw, scram_pw, "scram-sha-256", True),
        (None, "SCRAM-SHA-256$4096:AAAA$AAAA:AAAA", scram_pw, "scram-sha-256", False),
        (None, "SCRAM-SHA-256$foo", scram_pw, "scram-sha-256", False),
        (None, "password", "password", False, True),
        (None, "another", "password", False, False),
        (None, "password", "password", "foo", False),
        ("username", "password", md5_pw, "scram-sha-256", False),
        ("username", "password", scram_pw, "md5", False),
        # Code does not currently check role of pre-hashed md5 passwords
        pytest.param("another", md5_pw, md5_pw, "md5", False, marks=pytest.mark.xfail),
    ],
    ids=idfn,
)
def test_verify_password(role, password, verifier, method, result):
    assert postgres._verify_password(role, password, verifier, method) == result



#!/usr/bin/env python
"""
Deployment
==========

The ``rest_cherrypy`` netapi module is a standard Python WSGI app. It can be
deployed one of two ways.

:program:`salt-api` using the CherryPy server
---------------------------------------------

The default configuration is to run this module using :program:`salt-api` to
start the Python-based CherryPy server. This server is lightweight,
multi-threaded, encrypted with SSL, and should be considered production-ready.
See the section above for performance expectations.

Using a WSGI-compliant web server
---------------------------------

This module may be deployed on any WSGI-compliant server such as Apache with
mod_wsgi or Nginx with FastCGI, to name just two (there are many).

Note, external WSGI servers handle URLs, paths, and SSL certs directly. The
``rest_cherrypy`` configuration options are ignored and the ``salt-api`` daemon
does not need to be running at all. Remember Salt authentication credentials
are sent in the clear unless SSL is being enforced!

An example Apache virtual host configuration::

    <VirtualHost *:80>
        ServerName example.com
        ServerAlias *.example.com

        ServerAdmin webmaster@example.com

        LogLevel warn
        ErrorLog /var/www/example.com/logs/error.log
        CustomLog /var/www/example.com/logs/access.log combined

        DocumentRoot /var/www/example.com/htdocs

        WSGIScriptAlias / /path/to/salt/netapi/rest_cherrypy/wsgi.py
    </VirtualHost>

"""

import os

import cherrypy  # pylint: disable=3rd-party-module-not-gated

# pylint: disable=C0103


def bootstrap_app():
    """
    Grab the opts dict of the master config by trying to import Salt
    """
    from salt.netapi.rest_cherrypy import app
    import salt.config

    __opts__ = salt.config.client_config(
        os.environ.get("SALT_MASTER_CONFIG", "/etc/salt/master")
    )
    return app.get_app(__opts__)


def get_application(*args):
    """
    Returns a WSGI application function. If you supply the WSGI app and config
    it will use that, otherwise it will try to obtain them from a local Salt
    installation
    """
    opts_tuple = args

    def wsgi_app(environ, start_response):
        root, _, conf = opts_tuple or bootstrap_app()
        cherrypy.config.update({"environment": "embedded"})

        cherrypy.tree.mount(root, "/", conf)
        return cherrypy.tree(environ, start_response)

    return wsgi_app


application = get_application()



"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""


import errno
import os

import salt.modules.puppet as puppet
import salt.utils.args
import salt.utils.files
from salt.exceptions import CommandExecutionError
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, mock_open, patch
from tests.support.unit import TestCase


class PuppetTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.puppet
    """

    def setup_loader_modules(self):
        return {puppet: {}}

    def test_run(self):
        """
        Test to execute a puppet run
        """
        mock = MagicMock(return_value={"A": "B"})
        with patch.object(salt.utils.args, "clean_kwargs", mock):
            mock = MagicMock(return_value={"retcode": 0})
            mock_lst = MagicMock(return_value=[])
            with patch.dict(
                puppet.__salt__, {"cmd.run_all": mock, "cmd.run": mock_lst}
            ):
                self.assertTrue(puppet.run())

    def test_noop(self):
        """
        Test to execute a puppet noop run
        """
        mock = MagicMock(return_value={"stderr": "A", "stdout": "B"})
        with patch.object(puppet, "run", mock):
            self.assertDictEqual(puppet.noop(), {"stderr": "A", "stdout": "B"})

    def test_enable(self):
        """
        Test to enable the puppet agent
        """
        mock_lst = MagicMock(return_value=[])
        with patch.dict(puppet.__salt__, {"cmd.run": mock_lst}):
            mock = MagicMock(return_value=True)
            with patch.object(os.path, "isfile", mock):
                mock = MagicMock(return_value=True)
                with patch.object(os, "remove", mock):
                    self.assertTrue(puppet.enable())
                with patch.object(os, "remove", MagicMock(side_effect=IOError)):
                    self.assertRaises(CommandExecutionError, puppet.enable)

            self.assertFalse(puppet.enable())

    def test_disable(self):
        """
        Test to disable the puppet agent
        """
        mock_lst = MagicMock(return_value=[])
        with patch.dict(puppet.__salt__, {"cmd.run": mock_lst}):
            mock = MagicMock(side_effect=[True, False])
            with patch.object(os.path, "isfile", mock):
                self.assertFalse(puppet.disable())

                with patch("salt.utils.files.fopen", mock_open()):
                    self.assertTrue(puppet.disable())

                try:
                    with patch("salt.utils.files.fopen", mock_open()) as m_open:
                        m_open.side_effect = IOError(13, "Permission denied:", "/file")
                        self.assertRaises(CommandExecutionError, puppet.disable)
                except StopIteration:
                    pass

    def test_status(self):
        """
        Test to display puppet agent status
        """
        mock_lst = MagicMock(return_value=[])
        with patch.dict(puppet.__salt__, {"cmd.run": mock_lst}):
            mock = MagicMock(side_effect=[True])
            with patch.object(os.path, "isfile", mock):
                self.assertEqual(puppet.status(), "Administratively disabled")

            mock = MagicMock(side_effect=[False, True])
            with patch.object(os.path, "isfile", mock):
                with patch("salt.utils.files.fopen", mock_open(read_data="1")):
                    mock = MagicMock(return_value=True)
                    with patch.object(os, "kill", mock):
                        self.assertEqual(puppet.status(), "Applying a catalog")

            mock = MagicMock(side_effect=[False, True])
            with patch.object(os.path, "isfile", mock):
                with patch("salt.utils.files.fopen", mock_open()):
                    mock = MagicMock(return_value=True)
                    with patch.object(os, "kill", mock):
                        self.assertEqual(puppet.status(), "Stale lockfile")

            mock = MagicMock(side_effect=[False, False, True])
            with patch.object(os.path, "isfile", mock):
                with patch("salt.utils.files.fopen", mock_open(read_data="1")):
                    mock = MagicMock(return_value=True)
                    with patch.object(os, "kill", mock):
                        self.assertEqual(puppet.status(), "Idle daemon")

            mock = MagicMock(side_effect=[False, False, True])
            with patch.object(os.path, "isfile", mock):
                with patch("salt.utils.files.fopen", mock_open()):
                    mock = MagicMock(return_value=True)
                    with patch.object(os, "kill", mock):
                        self.assertEqual(puppet.status(), "Stale pidfile")

            mock = MagicMock(side_effect=[False, False, False])
            with patch.object(os.path, "isfile", mock):
                self.assertEqual(puppet.status(), "Stopped")

    def test_summary(self):
        """
        Test to show a summary of the last puppet agent run
        """
        mock_lst = MagicMock(return_value=[])
        with patch.dict(puppet.__salt__, {"cmd.run": mock_lst}):
            with patch("salt.utils.files.fopen", mock_open(read_data="resources: 1")):
                self.assertDictEqual(puppet.summary(), {"resources": 1})

            permission_error = IOError(errno.EACCES, "Permission denied:", "/file")
            with patch(
                "salt.utils.files.fopen", mock_open(read_data=permission_error)
            ) as m_open:
                self.assertRaises(CommandExecutionError, puppet.summary)

    def test_plugin_sync(self):
        """
        Test to runs a plugin synch between the puppet master and agent
        """
        mock_lst = MagicMock(return_value=[])
        with patch.dict(puppet.__salt__, {"cmd.run": mock_lst}):
            mock_lst = MagicMock(side_effect=[False, True])
            with patch.dict(puppet.__salt__, {"cmd.run": mock_lst}):
                self.assertEqual(puppet.plugin_sync(), "")

                self.assertTrue(puppet.plugin_sync())

    def test_facts(self):
        """
        Test to run facter and return the results
        """
        mock = MagicMock(return_value={"retcode": 0, "stdout": "1\n2"})
        with patch.dict(puppet.__salt__, {"cmd.run_all": mock}):
            mock = MagicMock(side_effect=[["a", "b"], ["c", "d"]])
            with patch.object(puppet, "_format_fact", mock):
                self.assertDictEqual(puppet.facts(), {"a": "b", "c": "d"})

    def test_fact(self):
        """
        Test to run facter for a specific fact
        """
        mock = MagicMock(
            side_effect=[
                {"retcode": 0, "stdout": False},
                {"retcode": 0, "stdout": True},
            ]
        )
        with patch.dict(puppet.__salt__, {"cmd.run_all": mock}):
            self.assertEqual(puppet.fact("salt"), "")

            self.assertTrue(puppet.fact("salt"))



"""
Manage Lambda Functions
=======================

.. versionadded:: 2016.3.0

Create and destroy Lambda Functions. Be aware that this interacts with Amazon's services,
and so may incur charges.

:depends:
    - boto
    - boto3

The dependencies listed above can be installed via package or pip.

This module accepts explicit vpc credentials but can also utilize
IAM roles assigned to the instance through Instance Profiles. Dynamic
credentials are then automatically obtained from AWS API and no further
configuration is necessary. More information available `here
<http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/iam-roles-for-amazon-ec2.html>`_.

If IAM roles are not used you need to specify them either in a pillar file or
in the minion's config file:

.. code-block:: yaml

    vpc.keyid: GKTADJGHEIQSXMKKRBJ08H
    vpc.key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs

It's also possible to specify ``key``, ``keyid`` and ``region`` via a profile,
either passed in as a dict, or as a string to pull from pillars or minion
config:

.. code-block:: yaml

    myprofile:
        keyid: GKTADJGHEIQSXMKKRBJ08H
        key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs
            region: us-east-1

.. code-block:: yaml

    Ensure function exists:
        boto_lambda.function_present:
            - FunctionName: myfunction
            - Runtime: python2.7
            - Role: iam_role_name
            - Handler: entry_function
            - ZipFile: code.zip
            - S3Bucket: bucketname
            - S3Key: keyname
            - S3ObjectVersion: version
            - Description: "My Lambda Function"
            - Timeout: 3
            - MemorySize: 128
            - region: us-east-1
            - keyid: GKTADJGHEIQSXMKKRBJ08H
            - key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs

"""


import hashlib
import logging
import os

import salt.utils.data
import salt.utils.dictupdate as dictupdate
import salt.utils.files
import salt.utils.json
from salt.exceptions import SaltInvocationError

log = logging.getLogger(__name__)


def __virtual__():
    """
    Only load if boto is available.
    """
    if "boto_lambda.function_exists" in __salt__:
        return "boto_lambda"
    return (False, "boto_lambda module could not be loaded")


def function_present(
    name,
    FunctionName,
    Runtime,
    Role,
    Handler,
    ZipFile=None,
    S3Bucket=None,
    S3Key=None,
    S3ObjectVersion=None,
    Description="",
    Timeout=3,
    MemorySize=128,
    Permissions=None,
    RoleRetries=5,
    region=None,
    key=None,
    keyid=None,
    profile=None,
    VpcConfig=None,
    Environment=None,
):
    """
    Ensure function exists.

    name
        The name of the state definition

    FunctionName
        Name of the Function.

    Runtime
        The Runtime environment for the function. One of
        'nodejs', 'java8', or 'python2.7'

    Role
        The name or ARN of the IAM role that the function assumes when it executes your
        function to access any other AWS resources.

    Handler
        The function within your code that Lambda calls to begin execution. For Node.js it is the
        module-name.*export* value in your function. For Java, it can be package.classname::handler or
        package.class-name.

    ZipFile
        A path to a .zip file containing your deployment package. If this is
        specified, S3Bucket and S3Key must not be specified.

    S3Bucket
        Amazon S3 bucket name where the .zip file containing your package is
        stored. If this is specified, S3Key must be specified and ZipFile must
        NOT be specified.

    S3Key
        The Amazon S3 object (the deployment package) key name you want to
        upload. If this is specified, S3Key must be specified and ZipFile must
        NOT be specified.

    S3ObjectVersion
        The version of S3 object to use. Optional, should only be specified if
        S3Bucket and S3Key are specified.

    Description
        A short, user-defined function description. Lambda does not use this value. Assign a meaningful
        description as you see fit.

    Timeout
        The function execution time at which Lambda should terminate this function. Because the execution
        time has cost implications, we recommend you set this value based on your expected execution time.
        The default is 3 seconds.

    MemorySize
        The amount of memory, in MB, your function is given. Lambda uses this memory size to infer
        the amount of CPU and memory allocated to your function. Your function use-case determines your
        CPU and memory requirements. For example, a database operation might need less memory compared
        to an image processing function. The default value is 128 MB. The value must be a multiple of
        64 MB.

    VpcConfig
        If your Lambda function accesses resources in a VPC, you must provide this parameter
        identifying the list of security group IDs/Names and subnet IDs/Name.  These must all belong
        to the same VPC.  This is a dict of the form:

        .. code-block:: yaml

            VpcConfig:
              SecurityGroupNames:
                - mysecgroup1
                - mysecgroup2
              SecurityGroupIds:
                - sg-abcdef1234
              SubnetNames:
                - mysubnet1
              SubnetIds:
                - subnet-1234abcd
                - subnet-abcd1234

        If VpcConfig is provided at all, you MUST pass at least one security group and one subnet.

    Permissions
        A list of permission definitions to be added to the function's policy

    RoleRetries
        IAM Roles may take some time to propagate to all regions once created.
        During that time function creation may fail; this state will
        atuomatically retry this number of times. The default is 5.

    Environment
        The parent object that contains your environment's configuration
        settings.  This is a dictionary of the form:

        .. code-block:: python

            {
                'Variables': {
                    'VariableName': 'VariableValue'
                }
            }

        .. versionadded:: 2017.7.0

    region
        Region to connect to.

    key
        Secret key to be used.

    keyid
        Access key to be used.

    profile
        A dict with region, key and keyid, or a pillar key (string) that
        contains a dict with region, key and keyid.
    """
    ret = {"name": FunctionName, "result": True, "comment": "", "changes": {}}

    if Permissions is not None:
        if isinstance(Permissions, str):
            Permissions = salt.utils.json.loads(Permissions)
        required_keys = {"Action", "Principal"}
        optional_keys = {"SourceArn", "SourceAccount", "Qualifier"}
        for sid, permission in Permissions.items():
            keyset = set(permission.keys())
            if not keyset.issuperset(required_keys):
                raise SaltInvocationError(
                    "{} are required for each permission specification".format(
                        ", ".join(required_keys)
                    )
                )
            keyset = keyset - required_keys
            keyset = keyset - optional_keys
            if bool(keyset):
                raise SaltInvocationError(
                    "Invalid permission value {}".format(", ".join(keyset))
                )

    r = __salt__["boto_lambda.function_exists"](
        FunctionName=FunctionName, region=region, key=key, keyid=keyid, profile=profile
    )

    if "error" in r:
        ret["result"] = False
        ret["comment"] = "Failed to create function: {}.".format(r["error"]["message"])
        return ret

    if not r.get("exists"):
        if __opts__["test"]:
            ret["comment"] = "Function {} is set to be created.".format(FunctionName)
            ret["result"] = None
            return ret
        r = __salt__["boto_lambda.create_function"](
            FunctionName=FunctionName,
            Runtime=Runtime,
            Role=Role,
            Handler=Handler,
            ZipFile=ZipFile,
            S3Bucket=S3Bucket,
            S3Key=S3Key,
            S3ObjectVersion=S3ObjectVersion,
            Description=Description,
            Timeout=Timeout,
            MemorySize=MemorySize,
            VpcConfig=VpcConfig,
            Environment=Environment,
            WaitForRole=True,
            RoleRetries=RoleRetries,
            region=region,
            key=key,
            keyid=keyid,
            profile=profile,
        )
        if not r.get("created"):
            ret["result"] = False
            ret["comment"] = "Failed to create function: {}.".format(
                r["error"]["message"]
            )
            return ret

        if Permissions:
            for sid, permission in Permissions.items():
                r = __salt__["boto_lambda.add_permission"](
                    FunctionName=FunctionName,
                    StatementId=sid,
                    region=region,
                    key=key,
                    keyid=keyid,
                    profile=profile,
                    **permission
                )
                if not r.get("updated"):
                    ret["result"] = False
                    ret["comment"] = "Failed to create function: {}.".format(
                        r["error"]["message"]
                    )

        _describe = __salt__["boto_lambda.describe_function"](
            FunctionName, region=region, key=key, keyid=keyid, profile=profile
        )
        _describe["function"]["Permissions"] = __salt__["boto_lambda.get_permissions"](
            FunctionName, region=region, key=key, keyid=keyid, profile=profile
        )["permissions"]
        ret["changes"]["old"] = {"function": None}
        ret["changes"]["new"] = _describe
        ret["comment"] = "Function {} created.".format(FunctionName)
        return ret

    ret["comment"] = os.linesep.join(
        [ret["comment"], "Function {} is present.".format(FunctionName)]
    )
    ret["changes"] = {}
    # function exists, ensure config matches
    _ret = _function_config_present(
        FunctionName,
        Role,
        Handler,
        Description,
        Timeout,
        MemorySize,
        VpcConfig,
        Environment,
        region,
        key,
        keyid,
        profile,
        RoleRetries,
    )
    if not _ret.get("result"):
        ret["result"] = _ret.get("result", False)
        ret["comment"] = _ret["comment"]
        ret["changes"] = {}
        return ret
    ret["changes"] = dictupdate.update(ret["changes"], _ret["changes"])
    ret["comment"] = " ".join([ret["comment"], _ret["comment"]])
    _ret = _function_code_present(
        FunctionName,
        ZipFile,
        S3Bucket,
        S3Key,
        S3ObjectVersion,
        region,
        key,
        keyid,
        profile,
    )
    if not _ret.get("result"):
        ret["result"] = _ret.get("result", False)
        ret["comment"] = _ret["comment"]
        ret["changes"] = {}
        return ret
    ret["changes"] = dictupdate.update(ret["changes"], _ret["changes"])
    ret["comment"] = " ".join([ret["comment"], _ret["comment"]])
    _ret = _function_permissions_present(
        FunctionName, Permissions, region, key, keyid, profile
    )
    if not _ret.get("result"):
        ret["result"] = _ret.get("result", False)
        ret["comment"] = _ret["comment"]
        ret["changes"] = {}
        return ret
    ret["changes"] = dictupdate.update(ret["changes"], _ret["changes"])
    ret["comment"] = " ".join([ret["comment"], _ret["comment"]])
    return ret


def _get_role_arn(name, region=None, key=None, keyid=None, profile=None):
    if name.startswith("arn:aws:iam:"):
        return name

    account_id = __salt__["boto_iam.get_account_id"](
        region=region, key=key, keyid=keyid, profile=profile
    )
    return "arn:aws:iam::{}:role/{}".format(account_id, name)


def _resolve_vpcconfig(conf, region=None, key=None, keyid=None, profile=None):
    if isinstance(conf, str):
        conf = salt.utils.json.loads(conf)
    if not conf:
        # if the conf is None, we should explicitly set the VpcConfig to
        # {'SubnetIds': [], 'SecurityGroupIds': []} to take the lambda out of
        # the VPC it was in
        return {"SubnetIds": [], "SecurityGroupIds": []}
    if not isinstance(conf, dict):
        raise SaltInvocationError("VpcConfig must be a dict.")
    sns = [
        __salt__["boto_vpc.get_resource_id"](
            "subnet", s, region=region, key=key, keyid=keyid, profile=profile
        ).get("id")
        for s in conf.pop("SubnetNames", [])
    ]
    sgs = [
        __salt__["boto_secgroup.get_group_id"](
            s, region=region, key=key, keyid=keyid, profile=profile
        )
        for s in conf.pop("SecurityGroupNames", [])
    ]
    conf.setdefault("SubnetIds", []).extend(sns)
    conf.setdefault("SecurityGroupIds", []).extend(sgs)
    return conf


def _function_config_present(
    FunctionName,
    Role,
    Handler,
    Description,
    Timeout,
    MemorySize,
    VpcConfig,
    Environment,
    region,
    key,
    keyid,
    profile,
    RoleRetries,
):
    ret = {"result": True, "comment": "", "changes": {}}
    func = __salt__["boto_lambda.describe_function"](
        FunctionName, region=region, key=key, keyid=keyid, profile=profile
    )["function"]
    need_update = False
    options = {
        "Role": _get_role_arn(Role, region, key, keyid, profile),
        "Handler": Handler,
        "Description": Description,
        "Timeout": Timeout,
        "MemorySize": MemorySize,
    }

    for key, val in options.items():
        if func[key] != val:
            need_update = True
            ret["changes"].setdefault("old", {})[key] = func[key]
            ret["changes"].setdefault("new", {})[key] = val
    # VpcConfig returns the extra value 'VpcId' so do a special compare
    oldval = func.get("VpcConfig")
    if oldval is not None:
        oldval.pop("VpcId", None)
    fixed_VpcConfig = _resolve_vpcconfig(VpcConfig, region, key, keyid, profile)
    if __utils__["boto3.ordered"](oldval) != __utils__["boto3.ordered"](
        fixed_VpcConfig
    ):
        need_update = True
        ret["changes"].setdefault("new", {})["VpcConfig"] = fixed_VpcConfig
        ret["changes"].setdefault("old", {})["VpcConfig"] = func.get("VpcConfig")

    if Environment is not None:
        if func.get("Environment") != Environment:
            need_update = True
            ret["changes"].setdefault("new", {})["Environment"] = Environment
            ret["changes"].setdefault("old", {})["Environment"] = func.get(
                "Environment"
            )

    if need_update:
        ret["comment"] = os.linesep.join(
            [ret["comment"], "Function config to be modified"]
        )
        if __opts__["test"]:
            ret["comment"] = "Function {} set to be modified.".format(FunctionName)
            ret["result"] = None
            return ret
        _r = __salt__["boto_lambda.update_function_config"](
            FunctionName=FunctionName,
            Role=Role,
            Handler=Handler,
            Description=Description,
            Timeout=Timeout,
            MemorySize=MemorySize,
            VpcConfig=fixed_VpcConfig,
            Environment=Environment,
            region=region,
            key=key,
            keyid=keyid,
            profile=profile,
            WaitForRole=True,
            RoleRetries=RoleRetries,
        )
        if not _r.get("updated"):
            ret["result"] = False
            ret["comment"] = "Failed to update function: {}.".format(
                _r["error"]["message"]
            )
            ret["changes"] = {}
    return ret


def _function_code_present(
    FunctionName, ZipFile, S3Bucket, S3Key, S3ObjectVersion, region, key, keyid, profile
):
    ret = {"result": True, "comment": "", "changes": {}}
    func = __salt__["boto_lambda.describe_function"](
        FunctionName, region=region, key=key, keyid=keyid, profile=profile
    )["function"]
    update = False
    if ZipFile:
        if "://" in ZipFile:  # Looks like a remote URL to me...
            dlZipFile = __salt__["cp.cache_file"](path=ZipFile)
            if dlZipFile is False:
                ret["result"] = False
                ret["comment"] = "Failed to cache ZipFile `{}`.".format(ZipFile)
                return ret
            ZipFile = dlZipFile
        size = os.path.getsize(ZipFile)
        if size == func["CodeSize"]:
            sha = hashlib.sha256()
            with salt.utils.files.fopen(ZipFile, "rb") as f:
                sha.update(f.read())
            hashed = sha.digest().encode("base64").strip()
            if hashed != func["CodeSha256"]:
                update = True
        else:
            update = True
    else:
        # No way to judge whether the item in the s3 bucket is current without
        # downloading it. Cheaper to just request an update every time, and still
        # idempotent
        update = True
    if update:
        if __opts__["test"]:
            ret["comment"] = "Function {} set to be modified.".format(FunctionName)
            ret["result"] = None
            return ret
        ret["changes"]["old"] = {
            "CodeSha256": func["CodeSha256"],
            "CodeSize": func["CodeSize"],
        }
        func = __salt__["boto_lambda.update_function_code"](
            FunctionName,
            ZipFile,
            S3Bucket,
            S3Key,
            S3ObjectVersion,
            region=region,
            key=key,
            keyid=keyid,
            profile=profile,
        )
        if not func.get("updated"):
            ret["result"] = False
            ret["comment"] = "Failed to update function: {}.".format(
                func["error"]["message"]
            )
            ret["changes"] = {}
            return ret
        func = func["function"]
        if (
            func["CodeSha256"] != ret["changes"]["old"]["CodeSha256"]
            or func["CodeSize"] != ret["changes"]["old"]["CodeSize"]
        ):
            ret["comment"] = os.linesep.join(
                [ret["comment"], "Function code to be modified"]
            )
            ret["changes"]["new"] = {
                "CodeSha256": func["CodeSha256"],
                "CodeSize": func["CodeSize"],
            }
        else:
            del ret["changes"]["old"]
    return ret


def _function_permissions_present(
    FunctionName, Permissions, region, key, keyid, profile
):
    ret = {"result": True, "comment": "", "changes": {}}
    curr_permissions = __salt__["boto_lambda.get_permissions"](
        FunctionName, region=region, key=key, keyid=keyid, profile=profile
    ).get("permissions")
    if curr_permissions is None:
        curr_permissions = {}
    need_update = False
    diffs = salt.utils.data.compare_dicts(curr_permissions, Permissions or {})
    if bool(diffs):
        ret["comment"] = os.linesep.join(
            [ret["comment"], "Function permissions to be modified"]
        )
        if __opts__["test"]:
            ret["comment"] = "Function {} set to be modified.".format(FunctionName)
            ret["result"] = None
            return ret
        for sid, diff in diffs.items():
            if diff.get("old", "") != "":
                # There's a permssion that needs to be removed
                _r = __salt__["boto_lambda.remove_permission"](
                    FunctionName=FunctionName,
                    StatementId=sid,
                    region=region,
                    key=key,
                    keyid=keyid,
                    profile=profile,
                )
                ret["changes"].setdefault("new", {}).setdefault("Permissions", {})[
                    sid
                ] = {}
                ret["changes"].setdefault("old", {}).setdefault("Permissions", {})[
                    sid
                ] = diff["old"]
            if diff.get("new", "") != "":
                # New permission information needs to be added
                _r = __salt__["boto_lambda.add_permission"](
                    FunctionName=FunctionName,
                    StatementId=sid,
                    region=region,
                    key=key,
                    keyid=keyid,
                    profile=profile,
                    **diff["new"]
                )
                ret["changes"].setdefault("new", {}).setdefault("Permissions", {})[
                    sid
                ] = diff["new"]
                oldperms = (
                    ret["changes"].setdefault("old", {}).setdefault("Permissions", {})
                )
                if sid not in oldperms:
                    oldperms[sid] = {}
            if not _r.get("updated"):
                ret["result"] = False
                ret["comment"] = "Failed to update function: {}.".format(
                    _r["error"]["message"]
                )
                ret["changes"] = {}
    return ret


def function_absent(
    name, FunctionName, region=None, key=None, keyid=None, profile=None
):
    """
    Ensure function with passed properties is absent.

    name
        The name of the state definition.

    FunctionName
        Name of the function.

    region
        Region to connect to.

    key
        Secret key to be used.

    keyid
        Access key to be used.

    profile
        A dict with region, key and keyid, or a pillar key (string) that
        contains a dict with region, key and keyid.
    """

    ret = {"name": FunctionName, "result": True, "comment": "", "changes": {}}

    r = __salt__["boto_lambda.function_exists"](
        FunctionName, region=region, key=key, keyid=keyid, profile=profile
    )
    if "error" in r:
        ret["result"] = False
        ret["comment"] = "Failed to delete function: {}.".format(r["error"]["message"])
        return ret

    if r and not r["exists"]:
        ret["comment"] = "Function {} does not exist.".format(FunctionName)
        return ret

    if __opts__["test"]:
        ret["comment"] = "Function {} is set to be removed.".format(FunctionName)
        ret["result"] = None
        return ret
    r = __salt__["boto_lambda.delete_function"](
        FunctionName, region=region, key=key, keyid=keyid, profile=profile
    )
    if not r["deleted"]:
        ret["result"] = False
        ret["comment"] = "Failed to delete function: {}.".format(r["error"]["message"])
        return ret
    ret["changes"]["old"] = {"function": FunctionName}
    ret["changes"]["new"] = {"function": None}
    ret["comment"] = "Function {} deleted.".format(FunctionName)
    return ret


def alias_present(
    name,
    FunctionName,
    Name,
    FunctionVersion,
    Description="",
    region=None,
    key=None,
    keyid=None,
    profile=None,
):
    """
    Ensure alias exists.

    name
        The name of the state definition.

    FunctionName
        Name of the function for which you want to create an alias.

    Name
        The name of the alias to be created.

    FunctionVersion
        Function version for which you are creating the alias.

    Description
        A short, user-defined function description. Lambda does not use this value. Assign a meaningful
        description as you see fit.

    region
        Region to connect to.

    key
        Secret key to be used.

    keyid
        Access key to be used.

    profile
        A dict with region, key and keyid, or a pillar key (string) that
        contains a dict with region, key and keyid.
    """
    ret = {"name": Name, "result": True, "comment": "", "changes": {}}

    r = __salt__["boto_lambda.alias_exists"](
        FunctionName=FunctionName,
        Name=Name,
        region=region,
        key=key,
        keyid=keyid,
        profile=profile,
    )

    if "error" in r:
        ret["result"] = False
        ret["comment"] = "Failed to create alias: {}.".format(r["error"]["message"])
        return ret

    if not r.get("exists"):
        if __opts__["test"]:
            ret["comment"] = "Alias {} is set to be created.".format(Name)
            ret["result"] = None
            return ret
        r = __salt__["boto_lambda.create_alias"](
            FunctionName,
            Name,
            FunctionVersion,
            Description,
            region,
            key,
            keyid,
            profile,
        )
        if not r.get("created"):
            ret["result"] = False
            ret["comment"] = "Failed to create alias: {}.".format(r["error"]["message"])
            return ret
        _describe = __salt__["boto_lambda.describe_alias"](
            FunctionName, Name, region=region, key=key, keyid=keyid, profile=profile
        )
        ret["changes"]["old"] = {"alias": None}
        ret["changes"]["new"] = _describe
        ret["comment"] = "Alias {} created.".format(Name)
        return ret

    ret["comment"] = os.linesep.join(
        [ret["comment"], "Alias {} is present.".format(Name)]
    )
    ret["changes"] = {}
    _describe = __salt__["boto_lambda.describe_alias"](
        FunctionName, Name, region=region, key=key, keyid=keyid, profile=profile
    )["alias"]

    need_update = False
    options = {"FunctionVersion": FunctionVersion, "Description": Description}

    for key, val in options.items():
        if _describe[key] != val:
            need_update = True
            ret["changes"].setdefault("old", {})[key] = _describe[key]
            ret["changes"].setdefault("new", {})[key] = val
    if need_update:
        ret["comment"] = os.linesep.join(
            [ret["comment"], "Alias config to be modified"]
        )
        if __opts__["test"]:
            ret["comment"] = "Alias {} set to be modified.".format(Name)
            ret["result"] = None
            return ret
        _r = __salt__["boto_lambda.update_alias"](
            FunctionName=FunctionName,
            Name=Name,
            FunctionVersion=FunctionVersion,
            Description=Description,
            region=region,
            key=key,
            keyid=keyid,
            profile=profile,
        )
        if not _r.get("updated"):
            ret["result"] = False
            ret["comment"] = "Failed to update alias: {}.".format(
                _r["error"]["message"]
            )
            ret["changes"] = {}
    return ret


def alias_absent(
    name, FunctionName, Name, region=None, key=None, keyid=None, profile=None
):
    """
    Ensure alias with passed properties is absent.

    name
        The name of the state definition.

    FunctionName
        Name of the function.

    Name
        Name of the alias.

    region
        Region to connect to.

    key
        Secret key to be used.

    keyid
        Access key to be used.

    profile
        A dict with region, key and keyid, or a pillar key (string) that
        contains a dict with region, key and keyid.
    """

    ret = {"name": Name, "result": True, "comment": "", "changes": {}}

    r = __salt__["boto_lambda.alias_exists"](
        FunctionName, Name, region=region, key=key, keyid=keyid, profile=profile
    )
    if "error" in r:
        ret["result"] = False
        ret["comment"] = "Failed to delete alias: {}.".format(r["error"]["message"])
        return ret

    if r and not r["exists"]:
        ret["comment"] = "Alias {} does not exist.".format(Name)
        return ret

    if __opts__["test"]:
        ret["comment"] = "Alias {} is set to be removed.".format(Name)
        ret["result"] = None
        return ret
    r = __salt__["boto_lambda.delete_alias"](
        FunctionName, Name, region=region, key=key, keyid=keyid, profile=profile
    )
    if not r["deleted"]:
        ret["result"] = False
        ret["comment"] = "Failed to delete alias: {}.".format(r["error"]["message"])
        return ret
    ret["changes"]["old"] = {"alias": Name}
    ret["changes"]["new"] = {"alias": None}
    ret["comment"] = "Alias {} deleted.".format(Name)
    return ret


def _get_function_arn(name, region=None, key=None, keyid=None, profile=None):
    if name.startswith("arn:aws:lambda:"):
        return name

    account_id = __salt__["boto_iam.get_account_id"](
        region=region, key=key, keyid=keyid, profile=profile
    )
    if profile and "region" in profile:
        region = profile["region"]
    if region is None:
        region = "us-east-1"
    return "arn:aws:lambda:{}:{}:function:{}".format(region, account_id, name)


def event_source_mapping_present(
    name,
    EventSourceArn,
    FunctionName,
    StartingPosition,
    Enabled=True,
    BatchSize=100,
    region=None,
    key=None,
    keyid=None,
    profile=None,
):
    """
    Ensure event source mapping exists.

    name
        The name of the state definition.

    EventSourceArn
        The Amazon Resource Name (ARN) of the Amazon Kinesis or the Amazon
        DynamoDB stream that is the event source.

    FunctionName
        The Lambda function to invoke when AWS Lambda detects an event on the
        stream.

        You can specify an unqualified function name (for example, "Thumbnail")
        or you can specify Amazon Resource Name (ARN) of the function (for
        example, "arn:aws:lambda:us-west-2:account-id:function:ThumbNail"). AWS
        Lambda also allows you to specify only the account ID qualifier (for
        example, "account-id:Thumbnail"). Note that the length constraint
        applies only to the ARN. If you specify only the function name, it is
        limited to 64 character in length.

    StartingPosition
        The position in the stream where AWS Lambda should start reading.
        (TRIM_HORIZON | LATEST)

    Enabled
        Indicates whether AWS Lambda should begin polling the event source. By
        default, Enabled is true.

    BatchSize
        The largest number of records that AWS Lambda will retrieve from your
        event source at the time of invoking your function. Your function
        receives an event with all the retrieved records. The default is 100
        records.

    region
        Region to connect to.

    key
        Secret key to be used.

    keyid
        Access key to be used.

    profile
        A dict with region, key and keyid, or a pillar key (string) that
        contains a dict with region, key and keyid.
    """
    ret = {"name": None, "result": True, "comment": "", "changes": {}}

    r = __salt__["boto_lambda.event_source_mapping_exists"](
        EventSourceArn=EventSourceArn,
        FunctionName=FunctionName,
        region=region,
        key=key,
        keyid=keyid,
        profile=profile,
    )

    if "error" in r:
        ret["result"] = False
        ret["comment"] = "Failed to create event source mapping: {}.".format(
            r["error"]["message"]
        )
        return ret

    if not r.get("exists"):
        if __opts__["test"]:
            ret["comment"] = "Event source mapping {} is set to be created.".format(
                FunctionName
            )
            ret["result"] = None
            return ret
        r = __salt__["boto_lambda.create_event_source_mapping"](
            EventSourceArn=EventSourceArn,
            FunctionName=FunctionName,
            StartingPosition=StartingPosition,
            Enabled=Enabled,
            BatchSize=BatchSize,
            region=region,
            key=key,
            keyid=keyid,
            profile=profile,
        )
        if not r.get("created"):
            ret["result"] = False
            ret["comment"] = "Failed to create event source mapping: {}.".format(
                r["error"]["message"]
            )
            return ret
        _describe = __salt__["boto_lambda.describe_event_source_mapping"](
            EventSourceArn=EventSourceArn,
            FunctionName=FunctionName,
            region=region,
            key=key,
            keyid=keyid,
            profile=profile,
        )
        ret["name"] = _describe["event_source_mapping"]["UUID"]
        ret["changes"]["old"] = {"event_source_mapping": None}
        ret["changes"]["new"] = _describe
        ret["comment"] = "Event source mapping {} created.".format(ret["name"])
        return ret

    ret["comment"] = os.linesep.join(
        [ret["comment"], "Event source mapping is present."]
    )
    ret["changes"] = {}
    _describe = __salt__["boto_lambda.describe_event_source_mapping"](
        EventSourceArn=EventSourceArn,
        FunctionName=FunctionName,
        region=region,
        key=key,
        keyid=keyid,
        profile=profile,
    )["event_source_mapping"]

    need_update = False
    options = {"BatchSize": BatchSize}

    for key, val in options.items():
        if _describe[key] != val:
            need_update = True
            ret["changes"].setdefault("old", {})[key] = _describe[key]
            ret["changes"].setdefault("new", {})[key] = val
    # verify FunctionName against FunctionArn
    function_arn = _get_function_arn(
        FunctionName, region=region, key=key, keyid=keyid, profile=profile
    )
    if _describe["FunctionArn"] != function_arn:
        need_update = True
        ret["changes"].setdefault("new", {})["FunctionArn"] = function_arn
        ret["changes"].setdefault("old", {})["FunctionArn"] = _describe["FunctionArn"]
    # TODO check for 'Enabled', since it doesn't directly map to a specific
    # state
    if need_update:
        ret["comment"] = os.linesep.join(
            [ret["comment"], "Event source mapping to be modified"]
        )
        if __opts__["test"]:
            ret["comment"] = "Event source mapping {} set to be modified.".format(
                _describe["UUID"]
            )
            ret["result"] = None
            return ret
        _r = __salt__["boto_lambda.update_event_source_mapping"](
            UUID=_describe["UUID"],
            FunctionName=FunctionName,
            Enabled=Enabled,
            BatchSize=BatchSize,
            region=region,
            key=key,
            keyid=keyid,
            profile=profile,
        )
        if not _r.get("updated"):
            ret["result"] = False
            ret["comment"] = "Failed to update mapping: {}.".format(
                _r["error"]["message"]
            )
            ret["changes"] = {}
    return ret


def event_source_mapping_absent(
    name, EventSourceArn, FunctionName, region=None, key=None, keyid=None, profile=None
):
    """
    Ensure event source mapping with passed properties is absent.

    name
        The name of the state definition.

    EventSourceArn
        ARN of the event source.

    FunctionName
        Name of the lambda function.

    region
        Region to connect to.

    key
        Secret key to be used.

    keyid
        Access key to be used.

    profile
        A dict with region, key and keyid, or a pillar key (string) that
        contains a dict with region, key and keyid.
    """

    ret = {"name": None, "result": True, "comment": "", "changes": {}}

    desc = __salt__["boto_lambda.describe_event_source_mapping"](
        EventSourceArn=EventSourceArn,
        FunctionName=FunctionName,
        region=region,
        key=key,
        keyid=keyid,
        profile=profile,
    )
    if "error" in desc:
        ret["result"] = False
        ret["comment"] = "Failed to delete event source mapping: {}.".format(
            desc["error"]["message"]
        )
        return ret

    if not desc.get("event_source_mapping"):
        ret["comment"] = "Event source mapping does not exist."
        return ret

    ret["name"] = desc["event_source_mapping"]["UUID"]
    if __opts__["test"]:
        ret["comment"] = "Event source mapping is set to be removed."
        ret["result"] = None
        return ret
    r = __salt__["boto_lambda.delete_event_source_mapping"](
        EventSourceArn=EventSourceArn,
        FunctionName=FunctionName,
        region=region,
        key=key,
        keyid=keyid,
        profile=profile,
    )
    if not r["deleted"]:
        ret["result"] = False
        ret["comment"] = "Failed to delete event source mapping: {}.".format(
            r["error"]["message"]
        )
        return ret
    ret["changes"]["old"] = desc
    ret["changes"]["new"] = {"event_source_mapping": None}
    ret["comment"] = "Event source mapping deleted."
    return ret



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
import pytest
import salt.states.drac as drac
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {drac: {}}


def test_present():
    """
    Test to ensure the user exists on the Dell DRAC
    """
    name = "damian"
    password = "secret"
    permission = "login,test_alerts,clear_logs"

    ret = {"name": name, "result": True, "comment": "", "changes": {}}

    mock = MagicMock(return_value=[name])
    with patch.dict(drac.__salt__, {"drac.list_users": mock}):
        with patch.dict(drac.__opts__, {"test": True}):
            comt = "`{}` already exists".format(name)
            ret.update({"comment": comt})
            assert drac.present(name, password, permission) == ret

        with patch.dict(drac.__opts__, {"test": False}):
            comt = "`{}` already exists".format(name)
            ret.update({"comment": comt})
            assert drac.present(name, password, permission) == ret


def test_absent():
    """
    Test to ensure a user does not exist on the Dell DRAC
    """
    name = "damian"

    ret = {"name": name, "result": True, "comment": "", "changes": {}}

    mock = MagicMock(return_value=[])
    with patch.dict(drac.__salt__, {"drac.list_users": mock}):
        with patch.dict(drac.__opts__, {"test": True}):
            comt = "`{}` does not exist".format(name)
            ret.update({"comment": comt})
            assert drac.absent(name) == ret

        with patch.dict(drac.__opts__, {"test": False}):
            comt = "`{}` does not exist".format(name)
            ret.update({"comment": comt})
            assert drac.absent(name) == ret


def test_network():
    """
    Test to ensure the DRAC network settings are consistent
    """
    ip_ = "10.225.108.29"
    netmask = "255.255.255.224"
    gateway = "10.225.108.1"

    ret = {"name": ip_, "result": None, "comment": "", "changes": {}}

    net_info = {
        "IPv4 settings": {
            "IP Address": ip_,
            "Subnet Mask": netmask,
            "Gateway": gateway,
        }
    }

    mock_info = MagicMock(return_value=net_info)
    mock_bool = MagicMock(side_effect=[True, False])
    with patch.dict(
        drac.__salt__,
        {"drac.network_info": mock_info, "drac.set_network": mock_bool},
    ):
        with patch.dict(drac.__opts__, {"test": True}):
            assert drac.network(ip_, netmask, gateway) == ret

        with patch.dict(drac.__opts__, {"test": False}):
            comt = "Network is in the desired state"
            ret.update({"comment": comt, "result": True})
            assert drac.network(ip_, netmask, gateway) == ret

            comt = "unable to configure network"
            ret.update({"comment": comt, "result": False})
            assert drac.network(ip_, netmask, gateway) == ret



"""
Monitor temperature, humidity and pressure using the SenseHat of a Raspberry Pi
===============================================================================

.. versionadded:: 2017.7.0

:maintainer:    Benedikt Werner <1benediktwerner@gmail.com>
:maturity:      new
:depends:       sense_hat Python module
"""
import logging
import re

import salt.utils.beacons

log = logging.getLogger(__name__)


def __virtual__():
    return "sensehat.get_pressure" in __salt__


def validate(config):
    """
    Validate the beacon configuration
    """
    # Configuration for sensehat beacon should be a list
    if not isinstance(config, list):
        return False, "Configuration for sensehat beacon must be a list."
    else:
        config = salt.utils.beacons.list_to_dict(config)

        if "sensors" not in config:
            return False, "Configuration for sensehat beacon requires sensors."
    return True, "Valid beacon configuration"


def beacon(config):
    """
    Monitor the temperature, humidity and pressure using the SenseHat sensors.

    You can either specify a threshold for each value and only emit a beacon
    if it is exceeded or define a range and emit a beacon when the value is
    out of range.

    Units:
    * humidity:                     percent
    * temperature:                  degrees Celsius
    * temperature_from_pressure:    degrees Celsius
    * pressure:                     Millibars

    .. code-block:: yaml

        beacons:
          sensehat:
            - sensors:
                humidity: 70%
                temperature: [20, 40]
                temperature_from_pressure: 40
                pressure: 1500
    """
    ret = []
    min_default = {"humidity": "0", "pressure": "0", "temperature": "-273.15"}

    config = salt.utils.beacons.list_to_dict(config)

    for sensor in config.get("sensors", {}):
        sensor_function = "sensehat.get_{}".format(sensor)
        if sensor_function not in __salt__:
            log.error("No sensor for meassuring %s. Skipping.", sensor)
            continue

        sensor_config = config["sensors"][sensor]
        if isinstance(sensor_config, list):
            sensor_min = str(sensor_config[0])
            sensor_max = str(sensor_config[1])
        else:
            sensor_min = min_default.get(sensor, "0")
            sensor_max = str(sensor_config)

        if isinstance(sensor_min, str) and "%" in sensor_min:
            sensor_min = re.sub("%", "", sensor_min)
        if isinstance(sensor_max, str) and "%" in sensor_max:
            sensor_max = re.sub("%", "", sensor_max)
        sensor_min = float(sensor_min)
        sensor_max = float(sensor_max)

        current_value = __salt__[sensor_function]()
        if not sensor_min <= current_value <= sensor_max:
            ret.append({"tag": "sensehat/{}".format(sensor), sensor: current_value})

    return ret



import os
import textwrap

import pytest
import salt.utils.files
import salt.utils.platform
from tests.support.case import ModuleCase
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import skipIf


@skipIf(not salt.utils.platform.is_windows(), "windows test only")
@pytest.mark.windows_whitelisted
class WinPKGTest(ModuleCase):
    """
    Tests for salt.modules.win_pkg. There are already
    some pkg execution module tests in the test
    integration.modules.test_pkg but this will be for
    specific windows software repository tests while
    using the win_pkg module.
    """

    @classmethod
    def setUpClass(cls):
        cls.repo_dir = os.path.join(
            RUNTIME_VARS.FILES, "file", "base", "win", "repo-ng"
        )
        cls.curl_sls_path = os.path.join(cls.repo_dir, "curl.sls")

    def tearDown(self):
        if os.path.isfile(self.curl_sls_path):
            os.remove(self.curl_sls_path)

    @pytest.mark.destructive_test
    @pytest.mark.slow_test
    def test_adding_removing_pkg_sls(self):
        """
        Test add and removing a new pkg sls
        in the windows software repository
        """

        def _check_pkg(pkgs, check_refresh, exists=True):
            refresh = self.run_function("pkg.refresh_db")
            self.assertEqual(
                check_refresh,
                refresh["total"],
                msg="total returned {}. Expected return {}".format(
                    refresh["total"], check_refresh
                ),
            )
            repo_data = self.run_function("pkg.get_repo_data")
            repo_cache = os.path.join(
                RUNTIME_VARS.TMP, "rootdir", "cache", "files", "base", "win", "repo-ng"
            )
            for pkg in pkgs:
                if exists:
                    assert pkg in str(repo_data), str(repo_data)
                else:
                    assert pkg not in str(repo_data), str(repo_data)

                for root, dirs, files in os.walk(repo_cache):
                    if exists:
                        assert pkg + ".sls" in files
                    else:
                        assert pkg + ".sls" not in files

        pkgs = ["putty", "vlc"]
        # check putty and vlc are in cache and repo query
        _check_pkg(pkgs, 2)

        # now add new sls
        with salt.utils.files.fopen(self.curl_sls_path, "w") as fp_:
            fp_.write(
                textwrap.dedent(
                    """
                curl:
                  '7.46.0':
                    full_name: 'cURL'
                    {% if grains['cpuarch'] == 'AMD64' %}
                    installer: 'salt://win/repo-ng/curl/curl-7.46.0-win64.msi'
                    uninstaller: 'salt://win/repo-ng/curl/curl-7.46.0-win64.msi'
                    {% else %}
                    installer: 'salt://win/repo-ng/curl/curl-7.46.0-win32.msi'
                    uninstaller: 'salt://win/repo-ng/curl/curl-7.46.0-win32.msi'
                    {% endif %}
                    install_flags: '/qn /norestart'
                    uninstall_flags: '/qn /norestart'
                    msiexec: True
                    locale: en_US
                    reboot: False
                """
                )
            )
        # now check if curl is also in cache and repo query
        pkgs.append("curl")
        for pkg in pkgs:
            self.assertIn(pkg + ".sls", os.listdir(self.repo_dir))
        _check_pkg(pkgs, 3)

        # remove curl sls and check its not in cache and repo query
        os.remove(self.curl_sls_path)
        _check_pkg(["curl"], 2, exists=False)



"""
Sending Messages via SMTP
==========================

.. versionadded:: 2014.7.0

This state is useful for firing messages during state runs, using the SMTP
protocol

.. code-block:: yaml

    server-warning-message:
      smtp.send_msg:
        - name: 'This is a server warning message'
        - profile: my-smtp-account
        - recipient: admins@example.com
"""


def __virtual__():
    """
    Only load if the SMTP module is available in __salt__
    """
    if "smtp.send_msg" in __salt__:
        return "smtp"
    return (False, "smtp module could not be loaded")


def send_msg(
    name,
    recipient,
    subject,
    sender=None,
    profile=None,
    use_ssl="True",
    attachments=None,
):
    """
    Send a message via SMTP

    .. code-block:: yaml

        server-warning-message:
          smtp.send_msg:
            - name: 'This is a server warning message'
            - profile: my-smtp-account
            - subject: 'Message from Salt'
            - recipient: admin@example.com
            - sender: admin@example.com
            - use_ssl: True
            - attachments:
                - /var/log/syslog
                - /var/log/messages

    name
        The message to send via SMTP
    """
    ret = {"name": name, "changes": {}, "result": None, "comment": ""}

    if profile is None and sender is None:
        ret["result"] = False
        ret["comment"] = "Missing parameter sender or profile for state smtp.send_msg"
        return ret

    if __opts__["test"]:
        ret["comment"] = "Need to send message to {}: {}".format(
            recipient,
            name,
        )
        return ret
    command = __salt__["smtp.send_msg"](
        message=name,
        recipient=recipient,
        profile=profile,
        subject=subject,
        sender=sender,
        use_ssl=use_ssl,
        attachments=attachments,
    )

    if command:
        ret["result"] = True
        if attachments:
            atts = ", ".join(attachments)
            ret["comment"] = "Sent message to {0} with attachments ({2}): {1}".format(
                recipient, name, atts
            )
        else:
            ret["comment"] = "Sent message to {}: {}".format(recipient, name)
    else:
        ret["result"] = False
        ret["comment"] = "Unable to send message to {}: {}".format(recipient, name)
    return ret



"""
test for pillar file_tree.py
"""


import os
import shutil
import tempfile

import salt.pillar.file_tree as file_tree
import salt.utils.files
import salt.utils.stringutils
from tests.support.helpers import TstSuiteLoggingHandler
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import TestCase

MINION_ID = "test-host"
NODEGROUP_PATH = os.path.join("nodegroups", "test-group", "files")
HOST_PATH = os.path.join("hosts", MINION_ID, "files")

BASE_PILLAR_CONTENT = {"files": {"hostfile": b"base", "groupfile": b"base"}}
DEV_PILLAR_CONTENT = {
    "files": {
        "hostfile": b"base",
        "groupfile": b"dev2",
        "hostfile1": b"dev1",
        "groupfile1": b"dev1",
        "hostfile2": b"dev2",
    }
}
PARENT_PILLAR_CONTENT = {
    "files": {"hostfile": b"base", "groupfile": b"base", "hostfile2": b"dev2"}
}

FILE_DATA = {
    os.path.join("base", HOST_PATH, "hostfile"): "base",
    os.path.join("dev1", HOST_PATH, "hostfile1"): "dev1",
    os.path.join("dev2", HOST_PATH, "hostfile2"): "dev2",
    os.path.join("base", NODEGROUP_PATH, "groupfile"): "base",
    os.path.join("dev1", NODEGROUP_PATH, "groupfile1"): "dev1",
    os.path.join("dev2", NODEGROUP_PATH, "groupfile"): "dev2",  # test merging
}

_CHECK_MINIONS_RETURN = {"minions": [MINION_ID], "missing": []}


class FileTreePillarTestCase(TestCase, LoaderModuleMockMixin):
    "test file_tree pillar"
    maxDiff = None

    def setup_loader_modules(self):
        self.tmpdir = tempfile.mkdtemp(dir=RUNTIME_VARS.TMP)
        self.addCleanup(shutil.rmtree, self.tmpdir)
        cachedir = os.path.join(self.tmpdir, "cachedir")
        os.makedirs(os.path.join(cachedir, "file_tree"))
        self.pillar_path = self._create_pillar_files()
        return {
            file_tree: {
                "__opts__": {
                    "cachedir": cachedir,
                    "pillar_roots": {
                        "base": [os.path.join(self.pillar_path, "base")],
                        "dev": [
                            os.path.join(self.pillar_path, "base"),
                            os.path.join(self.pillar_path, "dev1"),
                            os.path.join(self.pillar_path, "dev2"),
                        ],
                        "parent": [
                            os.path.join(self.pillar_path, "base", "sub1"),
                            os.path.join(self.pillar_path, "dev2", "sub"),
                            os.path.join(self.pillar_path, "base", "sub2"),
                        ],
                    },
                    "pillarenv": "base",
                    "nodegroups": {"test-group": [MINION_ID]},
                    "optimization_order": [0, 1, 2],
                    "file_buffer_size": 262144,
                    "file_roots": {"base": "", "dev": "", "parent": ""},
                    "extension_modules": "",
                    "renderer": "yaml_jinja",
                    "renderer_blacklist": [],
                    "renderer_whitelist": [],
                }
            }
        }

    def _create_pillar_files(self):
        "create files in tempdir"
        pillar_path = os.path.join(self.tmpdir, "file_tree")
        for filename in FILE_DATA:
            filepath = os.path.join(pillar_path, filename)
            os.makedirs(os.path.dirname(filepath))
            with salt.utils.files.fopen(filepath, "w") as data_file:
                data_file.write(salt.utils.stringutils.to_str(FILE_DATA[filename]))
        return pillar_path

    def test_absolute_path(self):
        "check file tree is imported correctly with an absolute path"
        absolute_path = os.path.join(self.pillar_path, "base")
        with patch(
            "salt.utils.minions.CkMinions.check_minions",
            MagicMock(return_value=_CHECK_MINIONS_RETURN),
        ):
            mypillar = file_tree.ext_pillar(MINION_ID, None, absolute_path)
            self.assertEqual(BASE_PILLAR_CONTENT, mypillar)

            with patch.dict(file_tree.__opts__, {"pillarenv": "dev"}):
                mypillar = file_tree.ext_pillar(MINION_ID, None, absolute_path)
                self.assertEqual(BASE_PILLAR_CONTENT, mypillar)

    def test_relative_path(self):
        "check file tree is imported correctly with a relative path"
        with patch(
            "salt.utils.minions.CkMinions.check_minions",
            MagicMock(return_value=_CHECK_MINIONS_RETURN),
        ):
            mypillar = file_tree.ext_pillar(MINION_ID, None, ".")
            self.assertEqual(BASE_PILLAR_CONTENT, mypillar)

            with patch.dict(file_tree.__opts__, {"pillarenv": "dev"}):
                mypillar = file_tree.ext_pillar(MINION_ID, None, ".")
                self.assertEqual(DEV_PILLAR_CONTENT, mypillar)

    def test_parent_path(self):
        "check if file tree is merged correctly with a .. path"
        with patch(
            "salt.utils.minions.CkMinions.check_minions",
            MagicMock(return_value=_CHECK_MINIONS_RETURN),
        ):
            with patch.dict(file_tree.__opts__, {"pillarenv": "parent"}):
                mypillar = file_tree.ext_pillar(MINION_ID, None, "..")
                self.assertEqual(PARENT_PILLAR_CONTENT, mypillar)

    def test_no_pillarenv(self):
        "confirm that file_tree yells when pillarenv is missing for a relative path"
        with patch(
            "salt.utils.minions.CkMinions.check_minions",
            MagicMock(return_value=_CHECK_MINIONS_RETURN),
        ):
            with patch.dict(file_tree.__opts__, {"pillarenv": None}):
                with TstSuiteLoggingHandler() as handler:
                    mypillar = file_tree.ext_pillar(MINION_ID, None, ".")
                    self.assertEqual({}, mypillar)

                    for message in handler.messages:
                        if (
                            message.startswith("ERROR:")
                            and "pillarenv is not set" in message
                        ):
                            break
                    else:
                        raise AssertionError("Did not find error message")

    def test_file_tree_bytes(self):
        """
        test file_tree pillar returns bytes
        """
        absolute_path = os.path.join(self.pillar_path, "base")
        with patch(
            "salt.utils.minions.CkMinions.check_minions",
            MagicMock(return_value=_CHECK_MINIONS_RETURN),
        ):
            mypillar = file_tree.ext_pillar(MINION_ID, None, absolute_path)
            self.assertEqual(BASE_PILLAR_CONTENT, mypillar)

            with patch.dict(file_tree.__opts__, {"pillarenv": "dev"}):
                mypillar = file_tree.ext_pillar(MINION_ID, None, absolute_path)
                self.assertEqual(mypillar["files"]["groupfile"], b"base")



import salt.utils.stringutils
from salt.pillar.pillar_ldap import _config
from tests.support.unit import TestCase


class LdapPillarTestCase(TestCase):
    def test__config_returns_str(self):
        conf = {"foo": "bar"}
        assert _config("foo", conf) == salt.utils.stringutils.to_str("bar")

    def test__conf_defaults_to_none(self):
        conf = {"foo": "bar"}
        assert _config("bang", conf) is None

    def test__conf_returns_str_from_unicode_default(self):
        conf = {"foo": "bar"}
        default = salt.utils.stringutils.to_unicode("bam")
        assert _config("bang", conf, default) == salt.utils.stringutils.to_str("bam")



"""
    :codeauthor: :email: `Justin Anderson <janderson@saltstack.com>`

    tests.integration.states.network
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""

import pytest
from tests.support.case import ModuleCase
from tests.support.mixins import SaltReturnAssertsMixin


@pytest.mark.destructive_test
class NetworkTest(ModuleCase, SaltReturnAssertsMixin):
    """
    Validate network state module
    """

    def setUp(self):
        os_family = self.run_function("grains.get", ["os_family"])
        if os_family not in ("RedHat", "Debian"):
            self.skipTest(
                "Network state only supported on RedHat and Debian based systems"
            )

    @pytest.mark.slow_test
    def test_managed(self):
        """
        network.managed
        """
        state_key = "network_|-dummy0_|-dummy0_|-managed"

        ret = self.run_function("state.sls", mods="network.managed", test=True)
        self.assertEqual(
            "Interface dummy0 is set to be added.", ret[state_key]["comment"]
        )

    @pytest.mark.slow_test
    def test_routes(self):
        """
        network.routes
        """
        state_key = "network_|-routes_|-dummy0_|-routes"
        expected_changes = "Interface dummy0 routes are set to be added."

        ret = self.run_function("state.sls", mods="network.routes", test=True)

        self.assertEqual(
            ret[state_key]["comment"], "Interface dummy0 routes are set to be added."
        )

    @pytest.mark.slow_test
    def test_system(self):
        """
        network.system
        """
        state_key = "network_|-system_|-system_|-system"

        global_settings = self.run_function("ip.get_network_settings")
        ret = self.run_function("state.sls", mods="network.system", test=True)
        self.assertIn(
            "Global network settings are set to be {}".format(
                "added" if not global_settings else "updated"
            ),
            ret[state_key]["comment"],
        )



"""
    :codeauthor: Nicole Thomas <nicole@saltstack.com>
"""

import pytest
from salt.exceptions import CommandExecutionError
from tests.support.case import ModuleCase
from tests.support.helpers import runs_on

# Brew doesn't support local package installation - So, let's
# Grab some small packages available online for brew
ADD_PKG = "algol68g"
DEL_PKG = "acme"


@pytest.mark.skip_if_not_root
@runs_on(kernel="Darwin")
@pytest.mark.destructive_test
@pytest.mark.skip_if_binaries_missing("brew")
class BrewModuleTest(ModuleCase):
    """
    Integration tests for the brew module
    """

    @pytest.mark.slow_test
    def test_brew_install(self):
        """
        Tests the installation of packages
        """
        try:
            self.run_function("pkg.install", [ADD_PKG])
            pkg_list = self.run_function("pkg.list_pkgs")
            try:
                self.assertIn(ADD_PKG, pkg_list)
            except AssertionError:
                self.run_function("pkg.remove", [ADD_PKG])
                raise
        except CommandExecutionError:
            self.run_function("pkg.remove", [ADD_PKG])
            raise

    @pytest.mark.slow_test
    def test_remove(self):
        """
        Tests the removal of packages
        """
        try:
            # Install a package to delete - If unsuccessful, skip the test
            self.run_function("pkg.install", [DEL_PKG])
            pkg_list = self.run_function("pkg.list_pkgs")
            if DEL_PKG not in pkg_list:
                self.run_function("pkg.install", [DEL_PKG])
                self.skipTest("Failed to install a package to delete")

            # Now remove the installed package
            self.run_function("pkg.remove", [DEL_PKG])
            del_list = self.run_function("pkg.list_pkgs")
            self.assertNotIn(DEL_PKG, del_list)
        except CommandExecutionError:
            self.run_function("pkg.remove", [DEL_PKG])
            raise

    @pytest.mark.slow_test
    def test_version(self):
        """
        Test pkg.version for mac. Installs a package and then checks we can get
        a version for the installed package.
        """
        try:
            self.run_function("pkg.install", [ADD_PKG])
            pkg_list = self.run_function("pkg.list_pkgs")
            version = self.run_function("pkg.version", [ADD_PKG])
            try:
                self.assertTrue(
                    version,
                    msg="version: {} is empty, or other issue is present".format(
                        version
                    ),
                )
                self.assertIn(
                    ADD_PKG,
                    pkg_list,
                    msg="package: {} is not in the list of installed packages: {}".format(
                        ADD_PKG, pkg_list
                    ),
                )
                # make sure the version is accurate and is listed in the pkg_list
                self.assertIn(
                    version,
                    str(pkg_list[ADD_PKG]),
                    msg="The {} version: {} is not listed in the pkg_list: {}".format(
                        ADD_PKG, version, pkg_list[ADD_PKG]
                    ),
                )
            except AssertionError:
                self.run_function("pkg.remove", [ADD_PKG])
                raise
        except CommandExecutionError:
            self.run_function("pkg.remove", [ADD_PKG])
            raise

    @pytest.mark.slow_test
    def test_latest_version(self):
        """
        Test pkg.latest_version:
          - get the latest version available
          - install the package
          - get the latest version available
          - check that the latest version is empty after installing it
        """
        try:
            self.run_function("pkg.remove", [ADD_PKG])
            uninstalled_latest = self.run_function("pkg.latest_version", [ADD_PKG])

            self.run_function("pkg.install", [ADD_PKG])
            installed_latest = self.run_function("pkg.latest_version", [ADD_PKG])
            version = self.run_function("pkg.version", [ADD_PKG])
            try:
                self.assertTrue(isinstance(uninstalled_latest, str))
                self.assertEqual(installed_latest, version)
            except AssertionError:
                self.run_function("pkg.remove", [ADD_PKG])
                raise
        except CommandExecutionError:
            self.run_function("pkg.remove", [ADD_PKG])
            raise

    @pytest.mark.slow_test
    def test_refresh_db(self):
        """
        Integration test to ensure pkg.refresh_db works with brew
        """
        refresh_brew = self.run_function("pkg.refresh_db")
        self.assertTrue(refresh_brew)

    @pytest.mark.slow_test
    def test_list_upgrades(self):
        """
        Test pkg.list_upgrades: data is in the form {'name1': 'version1',
        'name2': 'version2', ... }
        """
        try:
            upgrades = self.run_function("pkg.list_upgrades")
            try:
                self.assertTrue(isinstance(upgrades, dict))
                if upgrades:
                    for name in upgrades:
                        self.assertTrue(isinstance(name, str))
                        self.assertTrue(isinstance(upgrades[name], str))
            except AssertionError:
                self.run_function("pkg.remove", [ADD_PKG])
                raise
        except CommandExecutionError:
            self.run_function("pkg.remove", [ADD_PKG])
            raise

    @pytest.mark.slow_test
    def test_info_installed(self):
        """
        Test pkg.info_installed: info returned has certain fields used by
        mac_brew.latest_version
        """
        try:
            self.run_function("pkg.install", [ADD_PKG])
            info = self.run_function("pkg.info_installed", [ADD_PKG])
            try:
                self.assertTrue(ADD_PKG in info)
                self.assertTrue("versions" in info[ADD_PKG])
                self.assertTrue("revision" in info[ADD_PKG])
                self.assertTrue("stable" in info[ADD_PKG]["versions"])
            except AssertionError:
                self.run_function("pkg.remove", [ADD_PKG])
                raise
        except CommandExecutionError:
            self.run_function("pkg.remove", [ADD_PKG])
            raise

    def tearDown(self):
        """
        Clean up after tests
        """
        pkg_list = self.run_function("pkg.list_pkgs")

        # Remove any installed packages
        if ADD_PKG in pkg_list:
            self.run_function("pkg.remove", [ADD_PKG])
        if DEL_PKG in pkg_list:
            self.run_function("pkg.remove", [DEL_PKG])



"""
    :codeauthor: Rupesh Tare <rupesht@saltstack.com>
"""


import salt.modules.logadm as logadm
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class LogadmTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.logadm
    """

    def setup_loader_modules(self):
        return {logadm: {}}

    def test_show_conf(self):
        """
        Test for Show parsed configuration
        """
        with patch.object(logadm, "_parse_conf", return_value=True):
            self.assertTrue(logadm.show_conf("conf_file"))

    def test_rotate(self):
        """
        Test for Set up pattern for logging.
        """
        with patch.dict(
            logadm.__salt__,
            {"cmd.run_all": MagicMock(return_value={"retcode": 1, "stderr": "stderr"})},
        ):
            self.assertEqual(
                logadm.rotate("name"),
                {"Output": "stderr", "Error": "Failed in adding log"},
            )

        with patch.dict(
            logadm.__salt__,
            {"cmd.run_all": MagicMock(return_value={"retcode": 0, "stderr": "stderr"})},
        ):
            self.assertEqual(logadm.rotate("name"), {"Result": "Success"})

    def test_remove(self):
        """
        Test for Remove log pattern from logadm
        """
        with patch.dict(
            logadm.__salt__,
            {"cmd.run_all": MagicMock(return_value={"retcode": 1, "stderr": "stderr"})},
        ):
            self.assertEqual(
                logadm.remove("name"),
                {
                    "Output": "stderr",
                    "Error": "Failure in removing log. Possibly already removed?",
                },
            )

        with patch.dict(
            logadm.__salt__,
            {"cmd.run_all": MagicMock(return_value={"retcode": 0, "stderr": "stderr"})},
        ):
            self.assertEqual(logadm.remove("name"), {"Result": "Success"})



"""
Test the napalm_formula execution module.
"""

import textwrap

import salt.modules.napalm_formula as napalm_formula
from salt.utils.immutabletypes import freeze
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class TestModulesNAPALMFormula(TestCase, LoaderModuleMockMixin):
    @classmethod
    def setUpClass(cls):
        cls.model = freeze(
            {
                "interfaces": {
                    "interface": {
                        "Ethernet1": {
                            "config": {
                                "name": "Ethernet1",
                                "description": "Interface Ethernet1",
                            },
                            "subinterfaces": {
                                "subinterface": {
                                    "0": {
                                        "config": {
                                            "index": 0,
                                            "description": "Subinterface Ethernet1.0",
                                        }
                                    },
                                    "100": {
                                        "config": {
                                            "index": 100,
                                            "description": "Subinterface Ethernet1.100",
                                        }
                                    },
                                    "900": {
                                        "config": {
                                            "index": 900,
                                            "description": "Subinterface Ethernet1.900",
                                        }
                                    },
                                }
                            },
                        },
                        "Ethernet2": {
                            "config": {
                                "name": "Ethernet2",
                                "description": "Interface Ethernet2",
                            },
                            "subinterfaces": {
                                "subinterface": {
                                    "400": {
                                        "config": {
                                            "index": 400,
                                            "description": "Subinterface Ethernet2.400",
                                        }
                                    }
                                }
                            },
                        },
                    }
                }
            }
        )

        cls.defaults = freeze(
            {
                "interfaces": {
                    "interface": {
                        "*": {
                            "config": {"mtu": 2048, "enabled": True},
                            "subinterfaces": {
                                "subinterface": {"*": {"config": {"enabled": True}}}
                            },
                        }
                    }
                }
            }
        )

    def setup_loader_modules(self):
        patcher = patch("salt.utils.napalm.is_proxy", MagicMock(return_value=True))
        patcher.start()
        self.addCleanup(patcher.stop)
        return {napalm_formula: {"__grains__": {"os": "eos"}}}

    def test_container_path(self):
        paths = [
            "interfaces:interface:Ethernet1:config",
            "interfaces:interface:Ethernet1:subinterfaces:subinterface:0:config",
            "interfaces:interface:Ethernet1:subinterfaces:subinterface:100:config",
            "interfaces:interface:Ethernet2:subinterfaces:subinterface:400:config",
            "interfaces:interface:Ethernet1:subinterfaces:subinterface:900:config",
            "interfaces:interface:Ethernet2:config",
        ]
        ret = napalm_formula.container_path(self.model.copy())
        self.assertEqual(set(ret), set(paths))

    def test_setval(self):
        dict_ = {"foo": {"bar": {"baz": True}}}
        self.assertEqual(dict_, napalm_formula.setval("foo:bar:baz", True))

    def test_defaults(self):
        expected_result = {
            "interfaces": {
                "interface": {
                    "Ethernet1": {
                        "config": {
                            "name": "Ethernet1",
                            "description": "Interface Ethernet1",
                            "mtu": 2048,
                            "enabled": True,
                        },
                        "subinterfaces": {
                            "subinterface": {
                                "0": {
                                    "config": {
                                        "index": 0,
                                        "description": "Subinterface Ethernet1.0",
                                        "enabled": True,
                                    }
                                },
                                "100": {
                                    "config": {
                                        "index": 100,
                                        "description": "Subinterface Ethernet1.100",
                                        "enabled": True,
                                    }
                                },
                                "900": {
                                    "config": {
                                        "index": 900,
                                        "description": "Subinterface Ethernet1.900",
                                        "enabled": True,
                                    }
                                },
                            }
                        },
                    },
                    "Ethernet2": {
                        "config": {
                            "name": "Ethernet2",
                            "description": "Interface Ethernet2",
                            "mtu": 2048,
                            "enabled": True,
                        },
                        "subinterfaces": {
                            "subinterface": {
                                "400": {
                                    "config": {
                                        "index": 400,
                                        "description": "Subinterface Ethernet2.400",
                                        "enabled": True,
                                    }
                                }
                            }
                        },
                    },
                }
            }
        }
        ret = napalm_formula.defaults(self.model.copy(), self.defaults.copy())
        self.assertEqual(ret, expected_result)

    def test_render_field(self):
        config = {"description": "Interface description"}
        ret = napalm_formula.render_field(config, "description", quotes=True)
        self.assertEqual(ret, 'description "Interface description"')

    def test_render_field_junos(self):
        config = {"description": "Interface description"}
        with patch.dict(napalm_formula.__grains__, {"os": "junos"}):
            ret = napalm_formula.render_field(config, "description", quotes=True)
            self.assertEqual(ret, 'description "Interface description";')

    def test_render_fields(self):
        config = {"mtu": 2048, "description": "Interface description"}
        expected_render = textwrap.dedent(
            '''\
                mtu "2048"
                description "Interface description"'''
        )
        ret = napalm_formula.render_fields(config, "mtu", "description", quotes=True)
        self.assertEqual(ret, expected_render)



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

import pytest
import salt.states.modjk_worker as modjk_worker
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {modjk_worker: {}}


def test_stop():
    """
    Test to stop the named worker from the lbn load balancers
     at the targeted minions.
    """
    name = "{{ grains['id'] }}"
    lbn = "application"
    target = "roles:balancer"

    ret = {"name": name, "result": False, "comment": "", "changes": {}}

    comt = "no servers answered the published command modjk.worker_status"
    mock = MagicMock(return_value=False)
    with patch.dict(modjk_worker.__salt__, {"publish.publish": mock}):
        ret.update({"comment": comt})
        assert modjk_worker.stop(name, lbn, target) == ret


def test_activate():
    """
    Test to activate the named worker from the lbn load balancers
     at the targeted minions.
    """
    name = "{{ grains['id'] }}"
    lbn = "application"
    target = "roles:balancer"

    ret = {"name": name, "result": False, "comment": "", "changes": {}}

    comt = "no servers answered the published command modjk.worker_status"
    mock = MagicMock(return_value=False)
    with patch.dict(modjk_worker.__salt__, {"publish.publish": mock}):
        ret.update({"comment": comt})
        assert modjk_worker.activate(name, lbn, target) == ret


def test_disable():
    """
    Test to disable the named worker from the lbn load balancers
     at the targeted minions.
    """
    name = "{{ grains['id'] }}"
    lbn = "application"
    target = "roles:balancer"

    ret = {"name": name, "result": False, "comment": "", "changes": {}}

    comt = "no servers answered the published command modjk.worker_status"
    mock = MagicMock(return_value=False)
    with patch.dict(modjk_worker.__salt__, {"publish.publish": mock}):
        ret.update({"comment": comt})
        assert modjk_worker.disable(name, lbn, target) == ret



import subprocess
import types

import pytest
import salt.client.ssh.shell as shell


@pytest.fixture
def keys(tmp_path):
    pub_key = tmp_path / "ssh" / "testkey.pub"
    priv_key = tmp_path / "ssh" / "testkey"
    return types.SimpleNamespace(pub_key=pub_key, priv_key=priv_key)


@pytest.mark.skip_on_windows(reason="Windows does not support salt-ssh")
@pytest.mark.skip_if_binaries_missing("ssh", "ssh-keygen", check_all=True)
def test_ssh_shell_key_gen(keys):
    """
    Test ssh key_gen
    """
    shell.gen_key(str(keys.priv_key))
    assert keys.priv_key.exists()
    assert keys.pub_key.exists()
    # verify there is not a passphrase set on key
    ret = subprocess.check_output(
        ["ssh-keygen", "-f", str(keys.priv_key), "-y"],
        timeout=30,
    )
    assert ret.decode().startswith("ssh-rsa")



"""
    Tests for xml module
"""


import os
import tempfile

from salt.modules import xml
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase

XML_STRING = """
    <root xmlns:foo="http://www.foo.org/" xmlns:bar="http://www.bar.org">
        <actors>
            <actor id="1">Christian Bale</actor>
            <actor id="2">Liam Neeson</actor>
            <actor id="3">Michael Caine</actor>
        </actors>
        <foo:singers>
            <foo:singer id="4">Tom Waits</foo:singer>
            <foo:singer id="5">B.B. King</foo:singer>
            <foo:singer id="6">Ray Charles</foo:singer>
        </foo:singers>
    </root>
    """


class XmlTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.xml
    """

    def setup_loader_modules(self):
        return {xml: {}}

    def test_get_value(self):
        """
        Verify xml.get_value
        """
        with tempfile.NamedTemporaryFile("w+", delete=False) as xml_file:
            xml_file.write(XML_STRING)
            xml_file.flush()

        xml_result = xml.get_value(xml_file.name, ".//actor[@id='2']")
        self.assertEqual(xml_result, "Liam Neeson")

        os.remove(xml_file.name)

    def test_set_value(self):
        """
        Verify xml.set_value
        """
        with tempfile.NamedTemporaryFile("w+", delete=False) as xml_file:
            xml_file.write(XML_STRING)
            xml_file.flush()

        xml_result = xml.set_value(
            xml_file.name, ".//actor[@id='2']", "Patrick Stewart"
        )
        assert xml_result is True

        xml_result = xml.get_value(xml_file.name, ".//actor[@id='2']")
        self.assertEqual(xml_result, "Patrick Stewart")

        os.remove(xml_file.name)

    def test_get_attribute(self):
        """
        Verify xml.get_attribute
        """
        with tempfile.NamedTemporaryFile("w+", delete=False) as xml_file:
            xml_file.write(XML_STRING)
            xml_file.flush()

        xml_result = xml.get_attribute(xml_file.name, ".//actor[@id='3']")
        self.assertEqual(xml_result, {"id": "3"})

        os.remove(xml_file.name)

    def test_set_attribute(self):
        """
        Verify xml.set_value
        """
        with tempfile.NamedTemporaryFile("w+", delete=False) as xml_file:
            xml_file.write(XML_STRING)
            xml_file.flush()

        xml_result = xml.set_attribute(
            xml_file.name, ".//actor[@id='3']", "edited", "uh-huh"
        )
        assert xml_result is True

        xml_result = xml.get_attribute(xml_file.name, ".//actor[@id='3']")
        self.assertEqual(xml_result, {"edited": "uh-huh", "id": "3"})

        os.remove(xml_file.name)



"""
ZMQ-specific functions
"""

import logging

from salt._compat import ipaddress
from salt.exceptions import SaltSystemExit

log = logging.getLogger(__name__)

try:
    import zmq
except ImportError:
    zmq = None
    log.debug("ZMQ module is not found")

ZMQDefaultLoop = None
ZMQ_VERSION_INFO = (-1, -1, -1)
LIBZMQ_VERSION_INFO = (-1, -1, -1)

try:
    if zmq:
        ZMQ_VERSION_INFO = tuple([int(v_el) for v_el in zmq.__version__.split(".")])
        LIBZMQ_VERSION_INFO = tuple(
            [int(v_el) for v_el in zmq.zmq_version().split(".")]
        )
except Exception:  # pylint: disable=broad-except
    log.exception("Error while getting LibZMQ/PyZMQ library version")


def check_ipc_path_max_len(uri):
    # The socket path is limited to 107 characters on Solaris and
    # Linux, and 103 characters on BSD-based systems.
    if zmq is None:
        return
    ipc_path_max_len = getattr(zmq, "IPC_PATH_MAX_LEN", 103)
    if ipc_path_max_len and len(uri) > ipc_path_max_len:
        raise SaltSystemExit(
            "The socket path is longer than allowed by OS. "
            "'{}' is longer than {} characters. "
            "Either try to reduce the length of this setting's "
            "path or switch to TCP; in the configuration file, "
            'set "ipc_mode: tcp".'.format(uri, ipc_path_max_len)
        )


def ip_bracket(addr):
    """
    Ensure IP addresses are URI-compatible - specifically, add brackets
    around IPv6 literals if they are not already present.
    """
    addr = str(addr)
    addr = addr.lstrip("[")
    addr = addr.rstrip("]")
    addr = ipaddress.ip_address(addr)
    return ("[{}]" if addr.version == 6 else "{}").format(addr)



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""


import logging
import tempfile

import salt.modules.file
import salt.modules.ilo as ilo
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase

log = logging.getLogger(__name__)


class IloTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.ilo
    """

    def setup_loader_modules(self):
        return {
            ilo: {
                "__opts__": {"cachedir": tempfile.gettempdir()},
                "__salt__": {"file.remove": salt.modules.file.remove},
            }
        }

    # '__execute_cmd' function tests: 1

    def test_execute_cmd(self):
        """
        Test if __execute_command opens the temporary file
        properly when calling global_settings.
        """
        mock_cmd_run = MagicMock(return_value={"retcode": 0, "stdout": ""})
        with patch.dict(ilo.__salt__, {"cmd.run_all": mock_cmd_run}):
            ret = ilo.global_settings()
            self.assertEqual(ret, True)

    # 'global_settings' function tests: 1

    def test_global_settings(self):
        """
        Test if it shows global_settings
        """
        with patch(
            "salt.modules.ilo.__execute_cmd",
            MagicMock(return_value={"Global Settings": {}}),
        ):
            self.assertDictEqual(ilo.global_settings(), {"Global Settings": {}})

    # 'set_http_port' function tests: 1

    def test_set_http_port(self):
        """
        Test if it configure the port HTTP should listen on
        """
        with patch.object(
            ilo,
            "global_settings",
            return_value={"Global Settings": {"HTTP_PORT": {"VALUE": 80}}},
        ):
            self.assertTrue(ilo.set_http_port())

        with patch.object(
            ilo,
            "global_settings",
            return_value={"Global Settings": {"HTTP_PORT": {"VALUE": 40}}},
        ):
            with patch.object(ilo, "__execute_cmd", return_value={"Set HTTP Port": {}}):
                self.assertDictEqual(ilo.set_http_port(), {"Set HTTP Port": {}})

    # 'set_https_port' function tests: 1

    def test_set_https_port(self):
        """
        Test if it configure the port HTTPS should listen on
        """
        with patch.object(
            ilo,
            "global_settings",
            return_value={"Global Settings": {"HTTP_PORT": {"VALUE": 443}}},
        ):
            self.assertTrue(ilo.set_https_port())

        with patch.object(
            ilo,
            "global_settings",
            return_value={"Global Settings": {"HTTP_PORT": {"VALUE": 80}}},
        ):
            with patch.object(
                ilo, "__execute_cmd", return_value={"Set HTTPS Port": {}}
            ):
                self.assertDictEqual(ilo.set_https_port(), {"Set HTTPS Port": {}})

    # 'enable_ssh' function tests: 1

    def test_enable_ssh(self):
        """
        Test if it enable the SSH daemon
        """
        with patch.object(
            ilo,
            "global_settings",
            return_value={"Global Settings": {"SSH_STATUS": {"VALUE": "Y"}}},
        ):
            self.assertTrue(ilo.enable_ssh())

        with patch.object(
            ilo,
            "global_settings",
            return_value={"Global Settings": {"SSH_STATUS": {"VALUE": "N"}}},
        ):
            with patch.object(ilo, "__execute_cmd", return_value={"Enable SSH": {}}):
                self.assertDictEqual(ilo.enable_ssh(), {"Enable SSH": {}})

    # 'disable_ssh' function tests: 1

    def test_disable_ssh(self):
        """
        Test if it disable the SSH daemon
        """
        with patch.object(
            ilo,
            "global_settings",
            return_value={"Global Settings": {"SSH_STATUS": {"VALUE": "N"}}},
        ):
            self.assertTrue(ilo.disable_ssh())

        with patch.object(
            ilo,
            "global_settings",
            return_value={"Global Settings": {"SSH_STATUS": {"VALUE": "Y"}}},
        ):
            with patch.object(ilo, "__execute_cmd", return_value={"Disable SSH": {}}):
                self.assertDictEqual(ilo.disable_ssh(), {"Disable SSH": {}})

    # 'set_ssh_port' function tests: 1

    def test_set_ssh_port(self):
        """
        Test if it enable SSH on a user defined port
        """
        with patch.object(
            ilo,
            "global_settings",
            return_value={"Global Settings": {"SSH_PORT": {"VALUE": 22}}},
        ):
            self.assertTrue(ilo.set_ssh_port())

        with patch.object(
            ilo,
            "global_settings",
            return_value={"Global Settings": {"SSH_PORT": {"VALUE": 20}}},
        ):
            with patch.object(
                ilo, "__execute_cmd", return_value={"Configure SSH Port": {}}
            ):
                self.assertDictEqual(ilo.set_ssh_port(), {"Configure SSH Port": {}})

    # 'set_ssh_key' function tests: 1

    def test_set_ssh_key(self):
        """
        Test if it configure SSH public keys for specific users
        """
        with patch(
            "salt.modules.ilo.__execute_cmd",
            MagicMock(return_value={"Import SSH Publickey": {}}),
        ):
            self.assertDictEqual(
                ilo.set_ssh_key("ssh-rsa AAAAB3Nza Salt"), {"Import SSH Publickey": {}}
            )

    # 'delete_ssh_key' function tests: 1

    def test_delete_ssh_key(self):
        """
        Test if it delete a users SSH key from the ILO
        """
        with patch(
            "salt.modules.ilo.__execute_cmd",
            MagicMock(return_value={"Delete user SSH key": {}}),
        ):
            self.assertDictEqual(
                ilo.delete_ssh_key("Salt"), {"Delete user SSH key": {}}
            )

    # 'list_users' function tests: 1

    def test_list_users(self):
        """
        Test if it list all users
        """
        with patch(
            "salt.modules.ilo.__execute_cmd", MagicMock(return_value={"All users": {}})
        ):
            self.assertDictEqual(ilo.list_users(), {"All users": {}})

    # 'list_users_info' function tests: 1

    def test_list_users_info(self):
        """
        Test if it List all users in detail
        """
        with patch(
            "salt.modules.ilo.__execute_cmd",
            MagicMock(return_value={"All users info": {}}),
        ):
            self.assertDictEqual(ilo.list_users_info(), {"All users info": {}})

    # 'create_user' function tests: 1

    def test_create_user(self):
        """
        Test if it create user
        """
        with patch(
            "salt.modules.ilo.__execute_cmd",
            MagicMock(return_value={"Create user": {}}),
        ):
            self.assertDictEqual(
                ilo.create_user("Salt", "secretagent", "VIRTUAL_MEDIA_PRIV"),
                {"Create user": {}},
            )

    # 'delete_user' function tests: 1

    def test_delete_user(self):
        """
        Test if it delete a user
        """
        with patch(
            "salt.modules.ilo.__execute_cmd",
            MagicMock(return_value={"Delete user": {}}),
        ):
            self.assertDictEqual(ilo.delete_user("Salt"), {"Delete user": {}})

    # 'get_user' function tests: 1

    def test_get_user(self):
        """
        Test if it returns local user information, excluding the password
        """
        with patch(
            "salt.modules.ilo.__execute_cmd", MagicMock(return_value={"User Info": {}})
        ):
            self.assertDictEqual(ilo.get_user("Salt"), {"User Info": {}})

    # 'change_username' function tests: 1

    def test_change_username(self):
        """
        Test if it change a username
        """
        with patch(
            "salt.modules.ilo.__execute_cmd",
            MagicMock(return_value={"Change username": {}}),
        ):
            self.assertDictEqual(
                ilo.change_username("Salt", "SALT"), {"Change username": {}}
            )

    # 'change_password' function tests: 1

    def test_change_password(self):
        """
        Test if it reset a users password
        """
        with patch(
            "salt.modules.ilo.__execute_cmd",
            MagicMock(return_value={"Change password": {}}),
        ):
            self.assertDictEqual(
                ilo.change_password("Salt", "saltpasswd"), {"Change password": {}}
            )

    # 'network' function tests: 1

    def test_network(self):
        """
        Test if it grab the current network settings
        """
        with patch(
            "salt.modules.ilo.__execute_cmd",
            MagicMock(return_value={"Network Settings": {}}),
        ):
            self.assertDictEqual(ilo.network(), {"Network Settings": {}})

    # 'configure_network' function tests: 1

    def test_configure_network(self):
        """
        Test if it configure Network Interface
        """
        with patch(
            "salt.modules.ilo.__execute_cmd",
            MagicMock(return_value={"Configure_Network": {}}),
        ):
            ret = {
                "Network Settings": {
                    "IP_ADDRESS": {"VALUE": "10.0.0.10"},
                    "SUBNET_MASK": {"VALUE": "255.255.255.0"},
                    "GATEWAY_IP_ADDRESS": {"VALUE": "10.0.0.1"},
                }
            }
            with patch.object(ilo, "network", return_value=ret):
                self.assertTrue(
                    ilo.configure_network("10.0.0.10", "255.255.255.0", "10.0.0.1")
                )

            with patch.object(ilo, "network", return_value=ret):
                with patch.object(
                    ilo, "__execute_cmd", return_value={"Network Settings": {}}
                ):
                    self.assertDictEqual(
                        ilo.configure_network(
                            "10.0.0.100", "255.255.255.10", "10.0.0.10"
                        ),
                        {"Network Settings": {}},
                    )

    # 'enable_dhcp' function tests: 1

    def test_enable_dhcp(self):
        """
        Test if it enable DHCP
        """
        with patch.object(
            ilo,
            "network",
            return_value={"Network Settings": {"DHCP_ENABLE": {"VALUE": "Y"}}},
        ):
            self.assertTrue(ilo.enable_dhcp())

        with patch.object(
            ilo,
            "network",
            return_value={"Network Settings": {"DHCP_ENABLE": {"VALUE": "N"}}},
        ):
            with patch.object(ilo, "__execute_cmd", return_value={"Enable DHCP": {}}):
                self.assertDictEqual(ilo.enable_dhcp(), {"Enable DHCP": {}})

    # 'disable_dhcp' function tests: 1

    def test_disable_dhcp(self):
        """
        Test if it disable DHCP
        """
        with patch.object(
            ilo,
            "network",
            return_value={"Network Settings": {"DHCP_ENABLE": {"VALUE": "N"}}},
        ):
            self.assertTrue(ilo.disable_dhcp())

        with patch.object(
            ilo,
            "network",
            return_value={"Network Settings": {"DHCP_ENABLE": {"VALUE": "Y"}}},
        ):
            with patch.object(ilo, "__execute_cmd", return_value={"Disable DHCP": {}}):
                self.assertDictEqual(ilo.disable_dhcp(), {"Disable DHCP": {}})

    # 'configure_snmp' function tests: 1

    def test_configure_snmp(self):
        """
        Test if it configure SNMP
        """
        with patch(
            "salt.modules.ilo.__execute_cmd",
            MagicMock(return_value={"Configure SNMP": {}}),
        ):
            self.assertDictEqual(ilo.configure_snmp("Salt"), {"Configure SNMP": {}})



import copy

import salt.utils.json
from salt.cloud.clouds import scaleway
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class DummyRequest:
    def __init__(self, status_code, **response):
        self.status_code = status_code
        self.response = response

    def __getitem__(self, item):
        if item == "status":
            return self.status_code
        elif item in self.response:
            return self.response[item]
        raise KeyError(item)


class ScalewayTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.cloud.clouds.scaleway
    """

    _profile = {
        "profile": "my_scaleway",
        "name": "foo",
        "driver": "scaleway",
        "token": "foobarbaz",
    }

    def setup_loader_modules(self):
        return {
            scaleway: {
                "__utils__": {},
                "__opts__": {
                    "providers": {"my_scaleway": {}},
                    "profiles": {"my_scaleway": {}},
                },
            }
        }

    def test_query(self):
        """
        Confirm that using a different root affects the HTTP query made
        """
        body = '{"result": "success"}'
        server_id = "foo"
        expected = salt.utils.json.loads(body)
        http_query = MagicMock(return_value=DummyRequest(200, body=body))
        utils_dunder = {"http.query": http_query}

        with patch.dict(scaleway.__utils__, utils_dunder):

            # Case 1: use default api_root
            profile = copy.copy(self._profile)
            with patch.object(scaleway, "get_configured_provider", lambda: profile):
                result = scaleway.query(server_id=server_id)
                assert result == expected, result
                http_query.assert_called_once_with(
                    "https://cp-par1.scaleway.com/servers/foo/",
                    data="{}",
                    headers={
                        "X-Auth-Token": "foobarbaz",
                        "User-Agent": "salt-cloud",
                        "Content-Type": "application/json",
                    },
                    method="GET",
                )

            # Case 2: api_root overridden in profile
            http_query.reset_mock()
            profile = copy.copy(self._profile)
            profile["api_root"] = "https://my.api.root"
            with patch.object(scaleway, "get_configured_provider", lambda: profile):
                result = scaleway.query(server_id=server_id)
                assert result == expected, result
                http_query.assert_called_once_with(
                    "https://my.api.root/servers/foo/",
                    data="{}",
                    headers={
                        "X-Auth-Token": "foobarbaz",
                        "User-Agent": "salt-cloud",
                        "Content-Type": "application/json",
                    },
                    method="GET",
                )

            # Case 3: use default alternative root
            http_query.reset_mock()
            profile = copy.copy(self._profile)
            with patch.object(scaleway, "get_configured_provider", lambda: profile):
                result = scaleway.query(server_id=server_id, root="alt_root")
                assert result == expected, result
                http_query.assert_called_once_with(
                    "https://api-marketplace.scaleway.com/servers/foo/",
                    data="{}",
                    headers={
                        "X-Auth-Token": "foobarbaz",
                        "User-Agent": "salt-cloud",
                        "Content-Type": "application/json",
                    },
                    method="GET",
                )

            # Case 4: use alternative root specified in profile
            http_query.reset_mock()
            profile = copy.copy(self._profile)
            profile["alt_root"] = "https://my.alt.api.root"
            with patch.object(scaleway, "get_configured_provider", lambda: profile):
                result = scaleway.query(server_id=server_id, root="alt_root")
                assert result == expected, result
                http_query.assert_called_once_with(
                    "https://my.alt.api.root/servers/foo/",
                    data="{}",
                    headers={
                        "X-Auth-Token": "foobarbaz",
                        "User-Agent": "salt-cloud",
                        "Content-Type": "application/json",
                    },
                    method="GET",
                )



"""
mac_power tests
"""

import salt.modules.mac_power as mac_power
from salt.exceptions import SaltInvocationError
from tests.support.unit import TestCase


class MacPowerTestCase(TestCase):
    """
    test mac_power execution module
    """

    def test_validate_sleep_valid_number(self):
        """
        test _validate_sleep function with valid number
        """
        self.assertEqual(mac_power._validate_sleep(179), 179)

    def test_validate_sleep_invalid_number(self):
        """
        test _validate_sleep function with invalid number
        """
        self.assertRaises(SaltInvocationError, mac_power._validate_sleep, 181)

    def test_validate_sleep_valid_string(self):
        """
        test _validate_sleep function with valid string
        """
        self.assertEqual(mac_power._validate_sleep("never"), "Never")
        self.assertEqual(mac_power._validate_sleep("off"), "Never")

    def test_validate_sleep_invalid_string(self):
        """
        test _validate_sleep function with invalid string
        """
        self.assertRaises(SaltInvocationError, mac_power._validate_sleep, "bob")

    def test_validate_sleep_bool_true(self):
        """
        test _validate_sleep function with True
        """
        self.assertRaises(SaltInvocationError, mac_power._validate_sleep, True)

    def test_validate_sleep_bool_false(self):
        """
        test _validate_sleep function with False
        """
        self.assertEqual(mac_power._validate_sleep(False), "Never")

    def test_validate_sleep_unexpected(self):
        """
        test _validate_sleep function with True
        """
        self.assertRaises(SaltInvocationError, mac_power._validate_sleep, 172.7)
